@@ -78,7 +78,7 @@ daemon_opts = [
 k8s_opts = [
     cfg.StrOpt('api_root',
                help=_("The root URL of the Kubernetes API"),
-               default=os.environ.get('K8S_API', 'http://localhost:8080')),
+               default=os.environ.get('K8S_API', 'https://localhost:6443')),
     cfg.StrOpt('ssl_client_crt_file',
                help=_("Absolute path to client cert to "
                       "connect to HTTPS K8S_API")),
@@ -259,6 +259,16 @@ octavia_defaults = [
                       "to the pool members. The options are: ROUND_ROBIN, "
                       "LEAST_CONNECTIONS, SOURCE_IP and SOURCE_IP_PORT."),
                default='ROUND_ROBIN'),
+    cfg.IntOpt('timeout_client_data',
+               help=_("Frontend client inactivity timeout in milliseconds. "
+                      "When set to 0, the default timeout value set by "
+                      "Octavia is used."),
+               default=0),
+    cfg.IntOpt('timeout_member_data',
+               help=_("Backend member inactivity timeout in milliseconds. "
+                      "When set to 0, the default timeout value set by "
+                      "Octavia is used."),
+               default=0),
 ]
 
 cache_defaults = [
