@@ -29,6 +29,7 @@ from kuryr_kubernetes.controller.drivers import base
 from kuryr_kubernetes import exceptions as k_exc
 from kuryr_kubernetes import utils
 
+
 CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
@@ -118,7 +119,10 @@ class LBaaSv2Driver(base.LBaaSDriver):
         # won't assume this dict is sorted.
         max_ver = 0, 0
         for version in versions:
-            v_tuple = versionutils.convert_version_to_tuple(version['version'])
+            if version.get('version') is None:
+                raise k_exc.UnreachableOctavia('Unable to reach Octavia API')
+            v_tuple = versionutils.convert_version_to_tuple(
+                version['version'])
             if v_tuple > max_ver:
                 max_ver = v_tuple
 
@@ -357,7 +361,8 @@ class LBaaSv2Driver(base.LBaaSDriver):
                 'network-policy' not in rule.get('description'))
 
     def ensure_listener(self, loadbalancer, protocol, port,
-                        service_type='ClusterIP'):
+                        service_type='ClusterIP', timeout_client_data=0,
+                        timeout_member_data=0):
         name = "%s:%s:%s" % (loadbalancer['name'], protocol, port)
         listener = {
             'name': name,
@@ -366,6 +371,10 @@ class LBaaSv2Driver(base.LBaaSDriver):
             'protocol': protocol,
             'port': port
         }
+        if timeout_client_data:
+            listener['timeout_client_data'] = timeout_client_data
+        if timeout_member_data:
+            listener['timeout_member_data'] = timeout_member_data
         try:
             result = self._ensure_provisioned(
                 loadbalancer, listener, self._create_listener,
@@ -552,10 +561,20 @@ class LBaaSv2Driver(base.LBaaSDriver):
             'protocol': listener['protocol'],
             'protocol_port': listener['port'],
         }
+        timeout_cli = listener.get('timeout_client_data')
+        timeout_mem = listener.get('timeout_member_data')
+        if timeout_cli:
+            request['timeout_client_data'] = timeout_cli
+        if timeout_mem:
+            request['timeout_member_data'] = timeout_mem
         self.add_tags('listener', request)
         lbaas = clients.get_loadbalancer_client()
         response = lbaas.create_listener(**request)
         listener['id'] = response.id
+        if timeout_cli:
+            listener['timeout_client_data'] = response.timeout_client_data
+        if timeout_mem:
+            listener['timeout_member_data'] = response.timeout_member_data
         return listener
 
     def _update_listener_acls(self, loadbalancer, listener_id, allowed_cidrs):
@@ -589,6 +608,8 @@ class LBaaSv2Driver(base.LBaaSDriver):
 
     def _find_listener(self, listener, loadbalancer):
         lbaas = clients.get_loadbalancer_client()
+        timeout_cli = listener.get('timeout_client_data')
+        timeout_mb = listener.get('timeout_member_data')
         response = lbaas.listeners(
             name=listener['name'],
             project_id=listener['project_id'],
@@ -596,6 +617,9 @@ class LBaaSv2Driver(base.LBaaSDriver):
             protocol=listener['protocol'],
             protocol_port=listener['port'])
 
+        request = {}
+        request['timeout_client_data'] = timeout_cli
+        request['timeout_member_data'] = timeout_mb
         try:
             os_listener = next(response)
             listener['id'] = os_listener.id
@@ -603,9 +627,23 @@ class LBaaSv2Driver(base.LBaaSDriver):
                 LOG.debug("Releasing listener %s", os_listener.id)
                 self.release_listener(loadbalancer, listener)
                 return None
+            if (timeout_cli and (
+                    os_listener.timeout_client_data != timeout_cli)) or (
+                        timeout_mb and (
+                            os_listener.timeout_member_data != timeout_mb)):
+                LOG.debug("Updating listener %s", os_listener.id)
+                n_listen = lbaas.update_listener(os_listener.id, **request)
+                listener['timeout_client_data'] = n_listen.timeout_client_data
+                listener['timeout_member_data'] = n_listen.timeout_member_data
+            elif not timeout_cli or not timeout_mb:
+                LOG.debug("Updating listener %s", os_listener.id)
+                lbaas.update_listener(os_listener.id, **request)
+
         except (KeyError, StopIteration):
             return None
-
+        except os_exc.SDKException:
+            LOG.error('Error when updating listener %s' % listener['id'])
+            raise k_exc.ResourceNotReady(listener['id'])
         return listener
 
     def _create_pool(self, pool):
@@ -797,7 +835,7 @@ class LBaaSv2Driver(base.LBaaSDriver):
                           {'status': status, 'lb': loadbalancer,
                            'rem': remaining})
 
-        raise k_exc.ResourceNotReady(loadbalancer)
+        raise k_exc.LoadBalancerNotReady(loadbalancer['id'], status)
 
     def _wait_for_deletion(self, loadbalancer, timeout,
                            interval=_LB_STS_POLL_FAST_INTERVAL):
