@@ -14,6 +14,7 @@
 #    under the License.
 
 import urllib
+import uuid
 
 import netaddr
 from openstack import exceptions as os_exc
@@ -37,7 +38,7 @@ CONF = cfg.CONF
 
 
 def get_network_id(subnets):
-    ids = list(set(net.id for net in subnets.values()))
+    ids = list({net.id for net in subnets.values()})
 
     if len(ids) != 1:
         raise k_exc.IntegrityError(
@@ -80,7 +81,14 @@ def get_vifs(pod):
 
 
 def is_host_network(pod):
-    return pod['status'].get('hostNetwork', False)
+    return pod['spec'].get('hostNetwork', False)
+
+
+def is_pod_scheduled(pod):
+    try:
+        return bool(pod['spec']['nodeName'])
+    except KeyError:
+        return False
 
 
 def get_pods(selector, namespace=None):
@@ -208,6 +216,18 @@ def create_security_group_rule(body):
         raise
 
 
+def check_tag_on_creation():
+    """Checks if Neutron supports tagging during bulk port creation.
+
+    :param os_net: Network proxy object from Openstacksdk.
+    :return: Boolean
+    """
+    os_net = clients.get_network_client()
+    extension = os_net.find_extension(
+            name_or_id='tag-ports-during-bulk-creation')
+    return bool(extension)
+
+
 def delete_security_group_rule(security_group_rule_id):
     os_net = clients.get_network_client()
     try:
@@ -229,7 +249,7 @@ def patch_kuryrnetworkpolicy_crd(crd, i_rules, e_rules):
             'egressSgRules': e_rules,
         }
 
-        kubernetes.patch_crd('spec', crd['metadata']['selfLink'], spec)
+        kubernetes.patch_crd('spec', utils.get_res_link(crd), spec)
     except k_exc.K8sResourceNotFound:
         LOG.debug('KuryrNetworkPolicy CRD not found %s', crd_name)
     except k_exc.K8sClientException:
@@ -242,13 +262,9 @@ def create_security_group_rule_body(
         ethertype='IPv4', cidr=None,
         description="Kuryr-Kubernetes NetPolicy SG rule", namespace=None,
         pods=None):
-    if not port_range_min:
-        port_range_min = 1
-        port_range_max = 65535
-    elif not port_range_max:
+
+    if port_range_min and not port_range_max:
         port_range_max = port_range_min
-    if not protocol:
-        protocol = 'TCP'
 
     if cidr and netaddr.IPNetwork(cidr).version == 6:
         ethertype = 'IPv6'
@@ -258,11 +274,13 @@ def create_security_group_rule_body(
             'ethertype': ethertype,
             'description': description,
             'direction': direction,
-            'protocol': protocol.lower(),
-            'port_range_min': port_range_min,
-            'port_range_max': port_range_max,
         }
     }
+    if port_range_min and port_range_max:
+        security_group_rule_body['sgRule']['port_range_min'] = port_range_min
+        security_group_rule_body['sgRule']['port_range_max'] = port_range_max
+    if protocol:
+        security_group_rule_body['sgRule']['protocol'] = protocol.lower()
     if cidr:
         security_group_rule_body['sgRule']['remote_ip_prefix'] = cidr
     if namespace:
@@ -305,7 +323,6 @@ def get_annotated_labels(resource, annotation_labels):
 
 
 def get_kuryrnetworkpolicy_crds(namespace=None):
-    kubernetes = clients.get_kubernetes_client()
 
     try:
         if namespace:
@@ -313,15 +330,33 @@ def get_kuryrnetworkpolicy_crds(namespace=None):
                 constants.K8S_API_CRD_NAMESPACES, namespace)
         else:
             knp_path = constants.K8S_API_CRD_KURYRNETWORKPOLICIES
-        knps = kubernetes.get(knp_path)
+        knps = get_k8s_resource(knp_path)
         LOG.debug("Returning KuryrNetworkPolicies %s", knps)
-    except k_exc.K8sResourceNotFound:
-        LOG.exception("KuryrNetworkPolicy CRD not found")
-        return []
     except k_exc.K8sClientException:
-        LOG.exception("Kubernetes Client Exception")
-        raise
-    return knps.get('items', [])
+        LOG.exception("Exception during fetch KuryrNetworkPolicies. Retrying.")
+        raise k_exc.ResourceNotReady(knp_path)
+    return knps
+
+
+def get_kuryrloadbalancer_crds(namespace=None):
+    if namespace:
+        klb_path = '{}/{}/kuryrloadbalancers'.format(
+            constants.K8S_API_CRD_KURYRLOADBALANCERS, namespace)
+    else:
+        klb_path = constants.K8S_API_CRD_KURYRLOADBALANCERS
+    klbs = get_k8s_resource(klb_path)
+    return klbs
+
+
+def get_k8s_resource(resource_path):
+    kubernetes = clients.get_kubernetes_client()
+    k8s_resource = {}
+    try:
+        k8s_resource = kubernetes.get(resource_path)
+    except k_exc.K8sResourceNotFound:
+        LOG.exception('Kubernetes CRD not found')
+        return []
+    return k8s_resource.get('items', [])
 
 
 def get_networkpolicies(namespace=None):
@@ -331,7 +366,7 @@ def get_networkpolicies(namespace=None):
     try:
         if namespace:
             np_path = '{}/{}/networkpolicies'.format(
-                constants.K8S_API_CRD_NAMESPACES, namespace)
+                constants.K8S_API_NETWORKING_NAMESPACES, namespace)
         else:
             np_path = constants.K8S_API_POLICIES
         nps = kubernetes.get(np_path)
@@ -344,6 +379,22 @@ def get_networkpolicies(namespace=None):
     return nps.get('items', [])
 
 
+def zip_resources(xs, ys):
+    """Returns tuples of resources matched by namespace and name.
+
+    :param xs: List of objects x, first level of iteration.
+    :param ys: List of objects y.
+    :return: List of tuples of matching (x, y)
+    """
+    pairs = []
+    for x in xs:
+        for y in ys:
+            if utils.get_res_unique_name(x) == utils.get_res_unique_name(y):
+                pairs.append((x, y))
+                break
+    return pairs
+
+
 def zip_knp_np(knps, nps):
     """Returns tuples of matching KuryrNetworkPolicy and NetworkPolicy objs.
 
@@ -351,13 +402,7 @@ def zip_knp_np(knps, nps):
     :param nps: List of NetworkPolicy objects
     :return: List of tuples of matching (knp, np)
     """
-    pairs = []
-    for knp in knps:
-        for np in nps:
-            if utils.get_res_unique_name(knp) == utils.get_res_unique_name(np):
-                pairs.append((knp, np))
-                break
-    return pairs
+    return zip_resources(knps, nps)
 
 
 def match_expressions(expressions, labels):
@@ -577,3 +622,61 @@ def get_port_annot_pci_info(nodename, neutron_port):
         LOG.exception('Exception when reading annotations '
                       '%s and converting from json', annot_name)
     return pci_info
+
+
+def get_endpoints_targets(name, namespace):
+    kubernetes = clients.get_kubernetes_client()
+    target_ips = []
+    try:
+        klb_crd = kubernetes.get(
+            f'{constants.K8S_API_CRD_NAMESPACES}/{namespace}/'
+            f'kuryrloadbalancers/{name}')
+    except k_exc.K8sResourceNotFound:
+        LOG.debug("KuryrLoadBalancer %s not found on Namespace %s.",
+                  name, namespace)
+        return target_ips
+    except k_exc.K8sClientException:
+        LOG.exception('Exception when getting K8s Endpoints.')
+        raise
+
+    for ep_slice in klb_crd['spec'].get('endpointSlices', []):
+        for endpoint in ep_slice.get('endpoints', []):
+            target_ips.extend(endpoint.get('addresses', []))
+    return target_ips
+
+
+def bump_networkpolicy(knp):
+    kubernetes = clients.get_kubernetes_client()
+
+    try:
+        kubernetes.annotate(
+            knp['metadata']['annotations']['networkPolicyLink'],
+            {constants.K8S_ANNOTATION_POLICY: str(uuid.uuid4())})
+    except k_exc.K8sResourceNotFound:
+        raise
+    except k_exc.K8sClientException:
+        LOG.exception("Failed to annotate network policy %s to force its "
+                      "recalculation.", utils.get_res_unique_name(knp))
+        raise
+
+
+def bump_networkpolicies(namespace=None):
+    k8s = clients.get_kubernetes_client()
+    nps = get_networkpolicies(namespace)
+    for np in nps:
+        try:
+            k8s.annotate(utils.get_res_link(np),
+                         {constants.K8S_ANNOTATION_POLICY: str(uuid.uuid4())})
+        except k_exc.K8sResourceNotFound:
+            # Ignore if NP got deleted.
+            pass
+        except k_exc.K8sClientException:
+            LOG.warning("Failed to annotate network policy %s to force its "
+                        "recalculation.", utils.get_res_unique_name(np))
+            continue
+
+
+def is_network_policy_enabled():
+    enabled_handlers = CONF.kubernetes.enabled_handlers
+    svc_sg_driver = CONF.kubernetes.service_security_groups_driver
+    return 'policy' in enabled_handlers and svc_sg_driver == 'policy'
