@@ -21,13 +21,16 @@ from kuryr_kubernetes import clients
 from kuryr_kubernetes import config
 from kuryr_kubernetes import constants as k_const
 from kuryr_kubernetes.controller.drivers import base as drv_base
+from kuryr_kubernetes.controller.drivers import utils as driver_utils
 from kuryr_kubernetes import exceptions as k_exc
 from kuryr_kubernetes.handlers import k8s_base
 from kuryr_kubernetes import utils
 
 LOG = logging.getLogger(__name__)
+CONF = config.CONF
 
 OCTAVIA_DEFAULT_PROVIDERS = ['octavia', 'amphora']
+CRD_RECONCILIATION_FREQUENCY = 600  # seconds
 
 
 class KuryrLoadBalancerHandler(k8s_base.ResourceEventHandler):
@@ -49,12 +52,24 @@ class KuryrLoadBalancerHandler(k8s_base.ResourceEventHandler):
         self._drv_service_pub_ip = drv_base.ServicePubIpDriver.get_instance()
         self._drv_svc_project = drv_base.ServiceProjectDriver.get_instance()
         self._drv_sg = drv_base.ServiceSecurityGroupsDriver.get_instance()
-        # Note(yboaron) LBaaS driver supports 'provider' parameter in
-        # Load Balancer creation flow.
-        # We need to set the requested load balancer provider
-        # according to 'endpoints_driver_octavia_provider' configuration.
+        self._drv_nodes_subnets = drv_base.NodesSubnetsDriver.get_instance()
+
+    def _get_nodes_subnets(self):
+        return utils.get_subnets_id_cidrs(
+            self._drv_nodes_subnets.get_nodes_subnets())
+
+    def on_present(self, loadbalancer_crd, *args, **kwargs):
+        if loadbalancer_crd.get('status', None) is None:
+            kubernetes = clients.get_kubernetes_client()
+            try:
+                kubernetes.patch_crd('status',
+                                     utils.get_res_link(loadbalancer_crd),
+                                     {})
+            except k_exc.K8sResourceNotFound:
+                LOG.debug('KuryrLoadbalancer CRD not found %s',
+                          utils.get_res_unique_name(loadbalancer_crd))
+            return
 
-    def on_present(self, loadbalancer_crd):
         if self._should_ignore(loadbalancer_crd):
             LOG.debug("Ignoring Kubernetes service %s",
                       loadbalancer_crd['metadata']['name'])
@@ -76,17 +91,6 @@ class KuryrLoadBalancerHandler(k8s_base.ResourceEventHandler):
                         spec_lb_provider in OCTAVIA_DEFAULT_PROVIDERS):
                     self._ensure_release_lbaas(loadbalancer_crd)
 
-        try:
-            name = loadbalancer_crd['metadata']['name']
-            namespace = loadbalancer_crd['metadata']['namespace']
-            self._get_loadbalancer_crd(name, namespace)
-        except k_exc.K8sResourceNotFound:
-            LOG.debug('KuryrLoadbalancer CRD not found %s',
-                      loadbalancer_crd)
-        except KeyError:
-            LOG.debug('KuryrLoadbalancer CRD not found')
-            raise k_exc.ResourceNotReady(loadbalancer_crd)
-
         if self._sync_lbaas_members(loadbalancer_crd):
             # Note(yboaron) For LoadBalancer services, we should allocate FIP,
             # associate it to LB VIP and update K8S service status
@@ -108,39 +112,72 @@ class KuryrLoadBalancerHandler(k8s_base.ResourceEventHandler):
                     loadbalancer_crd['status'][
                         'service_pub_ip_info'] = service_pub_ip_info
                     self._update_lb_status(loadbalancer_crd)
-                    kubernetes = clients.get_kubernetes_client()
-                    try:
-                        kubernetes.patch_crd('status', loadbalancer_crd[
-                            'metadata']['selfLink'], loadbalancer_crd[
-                                'status'])
-                    except k_exc.K8sResourceNotFound:
-                        LOG.debug('KuryrLoadbalancer CRD not found %s',
-                                  loadbalancer_crd)
-                    except k_exc.K8sClientException:
-                        LOG.exception('Error updating KuryLoadbalancer CRD %s',
-                                      loadbalancer_crd)
-                        raise
+                    self._patch_status(loadbalancer_crd)
+
+    def reconcile(self):
+        loadbalancer_crds = []
+        try:
+            loadbalancer_crds = driver_utils.get_kuryrloadbalancer_crds()
+        except k_exc.K8sClientException:
+            LOG.warning("Error retriving KuryrLoadBalanders CRDs")
+        try:
+            self._trigger_loadbalancer_reconciliation(loadbalancer_crds)
+        except Exception:
+            LOG.exception('Error while running loadbalancers reconciliation.')
+
+    def _trigger_loadbalancer_reconciliation(self, loadbalancer_crds):
+        LOG.debug("Reconciling the loadbalancer CRDs")
+        # get the loadbalancers id in the CRD status
+        crd_loadbalancer_ids = [{'id': loadbalancer_crd.get('status', {}).get(
+                                'loadbalancer', {}).get('id', {}), 'selflink':
+                                utils.get_res_link(loadbalancer_crd)} for
+                                loadbalancer_crd in loadbalancer_crds]
+        lbaas = clients.get_loadbalancer_client()
+        lbaas_spec = {}
+        self._drv_lbaas.add_tags('loadbalancer', lbaas_spec)
+        loadbalancers = lbaas.load_balancers(**lbaas_spec)
+        loadbalancers_id = [loadbalancer['id']
+                            for loadbalancer in loadbalancers]
+        # for each loadbalancer id in the CRD status, check if exists in
+        # OpenStack
+        crds_to_reconcile_selflink = [crd_lb['selflink'] for crd_lb in
+                                      crd_loadbalancer_ids if
+                                      crd_lb['id'] not in loadbalancers_id]
+        if not crds_to_reconcile_selflink:
+            LOG.debug("KuryrLoadBalancer CRDs already in sync with OpenStack")
+            return
+        LOG.debug("Reconciling the following KuryrLoadBalancer CRDs: %r",
+                  crds_to_reconcile_selflink)
+        self._reconcile_lbaas(crds_to_reconcile_selflink)
+
+    def _reconcile_lbaas(self, crds_to_reconcile_selflink):
+        kubernetes = clients.get_kubernetes_client()
+        for selflink in crds_to_reconcile_selflink:
+            try:
+                kubernetes.patch_crd('status', selflink, {})
+            except k_exc.K8sResourceNotFound:
+                LOG.debug('Unable to reconcile the KuryLoadBalancer CRD %s',
+                          selflink)
+                continue
+            except k_exc.K8sClientException:
+                LOG.warning('Unable to patch the KuryLoadBalancer CRD %s',
+                            selflink)
+                continue
 
     def _should_ignore(self, loadbalancer_crd):
-        return not(self._has_pods(loadbalancer_crd) or
-                   loadbalancer_crd.get('status'))
+        return (not(self._has_endpoints(loadbalancer_crd) or
+                    loadbalancer_crd.get('status')) or not
+                loadbalancer_crd['spec'].get('ip'))
 
-    def _has_pods(self, loadbalancer_crd):
+    def _has_endpoints(self, loadbalancer_crd):
         ep_slices = loadbalancer_crd['spec'].get('endpointSlices', [])
         if not ep_slices:
             return False
-        return any(True
-                   for ep_slice in ep_slices
-                   for endpoint in ep_slice.get('endpoints', [])
-                   if endpoint['targetRef'].get('kind', []) == 'Pod')
+        return True
 
-    def on_finalize(self, loadbalancer_crd):
+    def on_finalize(self, loadbalancer_crd, *args, **kwargs):
         LOG.debug("Deleting the loadbalancer CRD")
 
-        if not loadbalancer_crd:
-            LOG.warning("Load Balancer CRD not present")
-            return
-
         if loadbalancer_crd['status'] != {}:
             # NOTE(ivc): deleting pool deletes its members
             self._drv_lbaas.release_loadbalancer(
@@ -172,8 +209,8 @@ class KuryrLoadBalancerHandler(k8s_base.ResourceEventHandler):
             service = kubernetes.get(f"{k_const.K8S_API_NAMESPACES}"
                                      f"/{namespace}/services/{name}")
         except k_exc.K8sResourceNotFound as ex:
-            LOG.exception("Failed to get service: %s", ex)
-            raise
+            LOG.warning("Failed to get service: %s", ex)
+            return
 
         LOG.debug('Removing finalizer from service %s',
                   service["metadata"]["name"])
@@ -184,18 +221,23 @@ class KuryrLoadBalancerHandler(k8s_base.ResourceEventHandler):
                           'for %s', service["metadata"]["name"])
             raise
 
-    def _get_loadbalancer_crd(self, loadbalancer_crd_name, namespace):
-        k8s = clients.get_kubernetes_client()
+    def _patch_status(self, loadbalancer_crd):
+        kubernetes = clients.get_kubernetes_client()
         try:
-            loadbalancer_crd = k8s.get('{}/{}/kuryrloadbalancers/{}'.format(
-                k_const.K8S_API_CRD_NAMESPACES, namespace,
-                loadbalancer_crd_name))
+            kubernetes.patch_crd('status', utils.get_res_link(
+                loadbalancer_crd), loadbalancer_crd['status'])
         except k_exc.K8sResourceNotFound:
-            return None
+            LOG.debug('KuryrLoadBalancer CRD not found %s', loadbalancer_crd)
+            return False
+        except k_exc.K8sUnprocessableEntity:
+            LOG.warning('KuryrLoadBalancer %s modified, retrying later.',
+                        utils.get_res_unique_name(loadbalancer_crd))
+            return False
         except k_exc.K8sClientException:
-            LOG.exception("Kubernetes Client Exception.")
+            LOG.exception('Error updating KuryLoadbalancer CRD %s',
+                          loadbalancer_crd)
             raise
-        return loadbalancer_crd
+        return True
 
     def _sync_lbaas_members(self, loadbalancer_crd):
         changed = False
@@ -206,7 +248,7 @@ class KuryrLoadBalancerHandler(k8s_base.ResourceEventHandler):
         if self._sync_lbaas_pools(loadbalancer_crd):
             changed = True
 
-        if (self._has_pods(loadbalancer_crd) and
+        if (self._has_endpoints(loadbalancer_crd) and
                 self._add_new_members(loadbalancer_crd)):
             changed = True
 
@@ -233,8 +275,7 @@ class KuryrLoadBalancerHandler(k8s_base.ResourceEventHandler):
         lb['security_groups'] = lb_sgs
 
         try:
-            k8s.patch_crd('status/loadbalancer',
-                          klb_crd['metadata']['selfLink'],
+            k8s.patch_crd('status/loadbalancer', utils.get_res_link(klb_crd),
                           {'security_groups': lb_sgs})
         except k_exc.K8sResourceNotFound:
             LOG.debug('KuryrLoadBalancer %s not found', svc_name)
@@ -244,8 +285,7 @@ class KuryrLoadBalancerHandler(k8s_base.ResourceEventHandler):
                       'due to missing loadbalancer field.')
             return None
         except k_exc.K8sClientException:
-            LOG.exception('Error syncing KuryrLoadBalancer'
-                          ' %s', svc_name)
+            LOG.exception('Error syncing KuryrLoadBalancer %s', svc_name)
             raise
         return klb_crd
 
@@ -286,9 +326,19 @@ class KuryrLoadBalancerHandler(k8s_base.ResourceEventHandler):
             for endpoint in ep_slice.get('endpoints', []):
                 try:
                     target_ip = endpoint['addresses'][0]
-                    target_ref = endpoint['targetRef']
-                    if target_ref['kind'] != k_const.K8S_OBJ_POD:
-                        continue
+                    target_ref = endpoint.get('targetRef')
+                    target_namespace = None
+                    if target_ref:
+                        target_namespace = target_ref['namespace']
+                    # Avoid to point to a Pod on hostNetwork
+                    # that isn't the one to be added as Member.
+                    if not target_ref and utils.get_subnet_by_ip(
+                            self._get_nodes_subnets(),
+                            target_ip):
+                        target_pod = {}
+                    else:
+                        target_pod = utils.get_pod_by_ip(
+                            target_ip, target_namespace)
                 except KeyError:
                     continue
                 if not pool_by_tgt_name:
@@ -304,25 +354,18 @@ class KuryrLoadBalancerHandler(k8s_base.ResourceEventHandler):
 
                     if (target_ip, target_port, pool['id']) in current_targets:
                         continue
-                    # TODO(apuimedo): Do not pass subnet_id at all when in
-                    # L3 mode once old neutron-lbaasv2 is not supported, as
-                    # octavia does not require it
-                    if (config.CONF.octavia_defaults.member_mode ==
-                            k_const.OCTAVIA_L2_MEMBER_MODE):
-                        try:
-                            member_subnet_id = self._get_pod_subnet(target_ref,
-                                                                    target_ip)
-                        except k_exc.K8sResourceNotFound:
-                            LOG.debug("Member namespace has been deleted. No "
-                                      "need to add the members as it is "
-                                      "going to be deleted")
-                            continue
-                    else:
-                        # We use the service subnet id so that the connectivity
-                        # from VIP to pods happens in layer 3 mode, i.e.,
-                        # routed.
-                        member_subnet_id = loadbalancer_crd['status'][
-                            'loadbalancer']['subnet_id']
+
+                    member_subnet_id = self._get_subnet_by_octavia_mode(
+                        target_pod, target_ip, loadbalancer_crd)
+
+                    if not member_subnet_id:
+                        LOG.warning("Skipping member creation for %s",
+                                    target_ip)
+                        continue
+
+                    target_name, target_namespace = self._get_target_info(
+                        target_ref, loadbalancer_crd)
+
                     first_member_of_the_pool = True
                     for member in loadbalancer_crd['status'].get(
                             'members', []):
@@ -341,8 +384,8 @@ class KuryrLoadBalancerHandler(k8s_base.ResourceEventHandler):
                         subnet_id=member_subnet_id,
                         ip=target_ip,
                         port=target_port,
-                        target_ref_namespace=target_ref['namespace'],
-                        target_ref_name=target_ref['name'],
+                        target_ref_namespace=target_namespace,
+                        target_ref_name=target_name,
                         listener_port=listener_port)
                     if not member:
                         continue
@@ -354,26 +397,42 @@ class KuryrLoadBalancerHandler(k8s_base.ResourceEventHandler):
                         loadbalancer_crd['status']['members'] = []
                         loadbalancer_crd['status'].get('members', []).append(
                             member)
-                    kubernetes = clients.get_kubernetes_client()
-                    try:
-                        kubernetes.patch_crd('status', loadbalancer_crd[
-                            'metadata']['selfLink'], loadbalancer_crd[
-                                'status'])
-                    except k_exc.K8sResourceNotFound:
-                        LOG.debug('KuryrLoadbalancer CRD not found %s',
-                                  loadbalancer_crd)
-                    except k_exc.K8sClientException:
-                        LOG.exception('Error updating KuryLoadbalancer CRD %s',
-                                      loadbalancer_crd)
-                        raise
+                    if not self._patch_status(loadbalancer_crd):
+                        return False
                     changed = True
         return changed
 
-    def _get_pod_subnet(self, target_ref, ip):
-        # REVISIT(ivc): consider using true pod object instead
-        pod = {'kind': target_ref['kind'],
-               'metadata': {'name': target_ref['name'],
-                            'namespace': target_ref['namespace']}}
+    def _get_target_info(self, target_ref, loadbalancer_crd):
+        if target_ref:
+            target_namespace = target_ref['namespace']
+            target_name = target_ref['name']
+        else:
+            target_namespace = loadbalancer_crd['metadata']['namespace']
+            target_name = loadbalancer_crd['metadata']['name']
+        return target_name, target_namespace
+
+    def _get_subnet_by_octavia_mode(self, target_pod, target_ip, lb_crd):
+        # TODO(apuimedo): Do not pass subnet_id at all when in
+        # L3 mode once old neutron-lbaasv2 is not supported, as
+        # octavia does not require it
+        subnet_id = None
+        if (CONF.octavia_defaults.member_mode ==
+                k_const.OCTAVIA_L2_MEMBER_MODE):
+            if target_pod:
+                subnet_id = self._get_pod_subnet(target_pod, target_ip)
+            else:
+                subnet = utils.get_subnet_by_ip(
+                    self._get_nodes_subnets(), target_ip)
+                if subnet:
+                    subnet_id = subnet[0]
+        else:
+            # We use the service subnet id so that the connectivity
+            # from VIP to pods happens in layer 3 mode, i.e.,
+            # routed.
+            subnet_id = lb_crd['status']['loadbalancer']['subnet_id']
+        return subnet_id
+
+    def _get_pod_subnet(self, pod, ip):
         project_id = self._drv_pod_project.get_project(pod)
         subnets_map = self._drv_pod_subnets.get_subnets(pod, project_id)
         subnet_ids = [subnet_id for subnet_id, network in subnets_map.items()
@@ -382,10 +441,16 @@ class KuryrLoadBalancerHandler(k8s_base.ResourceEventHandler):
         if subnet_ids:
             return subnet_ids[0]
         else:
-            # NOTE(ltomasbo): We are assuming that if ip is not on the
-            # pod subnet is because the member is using hostnetworking. In
-            # this worker_nodes_subnet will be used
-            return config.CONF.pod_vif_nested.worker_nodes_subnet
+            # NOTE(ltomasbo): We are assuming that if IP is not on the
+            # pod subnet it's because the member is using hostNetworking. In
+            # this case we look for the IP in worker_nodes_subnets.
+            subnet = utils.get_subnet_by_ip(self._get_nodes_subnets(), ip)
+            if subnet:
+                return subnet[0]
+            else:
+                # This shouldn't ever happen but let's return just the first
+                # worker_nodes_subnet id.
+                return self._get_nodes_subnets()[0][0]
 
     def _get_port_in_pool(self, pool, loadbalancer_crd):
 
@@ -399,6 +464,7 @@ class KuryrLoadBalancerHandler(k8s_base.ResourceEventHandler):
         return None
 
     def _remove_unused_members(self, loadbalancer_crd):
+        lb_crd_name = loadbalancer_crd['metadata']['name']
         spec_ports = {}
         pools = loadbalancer_crd['status'].get('pools', [])
         for pool in pools:
@@ -408,26 +474,24 @@ class KuryrLoadBalancerHandler(k8s_base.ResourceEventHandler):
                     port['name'] = None
                 spec_ports[port['name']] = pool['id']
 
-        ep_slices = loadbalancer_crd['spec'].get('endpointSlices')
-        # NOTE(maysams): As we don't support dual-stack, we assume
-        # only one address is possible on the addresses field.
-        current_targets = [(ep['addresses'][0],
-                           ep.get('targetRef', {}).get('name', ''),
-                           p['port'], spec_ports.get(p.get('name')))
+        ep_slices = loadbalancer_crd['spec'].get('endpointSlices', [])
+        current_targets = [utils.get_current_endpoints_target(
+                           ep, p, spec_ports, lb_crd_name)
                            for ep_slice in ep_slices
                            for ep in ep_slice['endpoints']
                            for p in ep_slice['ports']
                            if p.get('name') in spec_ports]
-        removed_ids = set()
 
+        removed_ids = set()
         for member in loadbalancer_crd['status'].get('members', []):
+            member_name = member.get('name', '')
             try:
-                member_name = member['name']
                 # NOTE: The member name is compose of:
                 # NAMESPACE_NAME/POD_NAME:PROTOCOL_PORT
                 pod_name = member_name.split('/')[1].split(':')[0]
             except AttributeError:
                 pod_name = ""
+
             if ((str(member['ip']), pod_name, member['port'], member[
                     'pool_id']) in current_targets):
                 continue
@@ -437,25 +501,12 @@ class KuryrLoadBalancerHandler(k8s_base.ResourceEventHandler):
             removed_ids.add(member['id'])
 
         if removed_ids:
-            loadbalancer_crd['status']['members'] = [m for m in
-                                                     loadbalancer_crd[
-                                                         'status'][
-                                                             'members']
-                                                     if m['id'] not in
-                                                     removed_ids]
+            members = [m for m in loadbalancer_crd['status'].get('members', [])
+                       if m['id'] not in removed_ids]
+            loadbalancer_crd['status']['members'] = members
 
-            kubernetes = clients.get_kubernetes_client()
-            try:
-                kubernetes.patch_crd('status', loadbalancer_crd[
-                    'metadata']['selfLink'], loadbalancer_crd[
-                        'status'])
-            except k_exc.K8sResourceNotFound:
-                LOG.debug('KuryrLoadbalancer CRD not found %s',
-                          loadbalancer_crd)
-            except k_exc.K8sClientException:
-                LOG.exception('Error updating KuryLoadbalancer CRD %s',
-                              loadbalancer_crd)
-                raise
+            if not self._patch_status(loadbalancer_crd):
+                return False
         return bool(removed_ids)
 
     def _sync_lbaas_pools(self, loadbalancer_crd):
@@ -493,17 +544,9 @@ class KuryrLoadBalancerHandler(k8s_base.ResourceEventHandler):
                 loadbalancer_crd['status']['pools'] = []
                 loadbalancer_crd['status'].get('pools', []).append(
                     pool)
-            kubernetes = clients.get_kubernetes_client()
-            try:
-                kubernetes.patch_crd('status', loadbalancer_crd['metadata'][
-                    'selfLink'], loadbalancer_crd['status'])
-            except k_exc.K8sResourceNotFound:
-                LOG.debug('KuryrLoadbalancer CRD not found %s',
-                          loadbalancer_crd)
-            except k_exc.K8sClientException:
-                LOG.exception('Error updating KuryrLoadbalancer CRD %s',
-                              loadbalancer_crd)
-                raise
+
+            if not self._patch_status(loadbalancer_crd):
+                return False
             changed = True
         return changed
 
@@ -532,25 +575,16 @@ class KuryrLoadBalancerHandler(k8s_base.ResourceEventHandler):
             removed_ids.add(pool['id'])
         if removed_ids:
             loadbalancer_crd['status']['pools'] = [p for p in loadbalancer_crd[
-                'status']['pools'] if p['id'] not in removed_ids]
+                'status'].get('pools', []) if p['id'] not in removed_ids]
             loadbalancer_crd['status']['members'] = [m for m in
                                                      loadbalancer_crd[
-                                                         'status']['members']
+                                                         'status'].get(
+                                                         'members', [])
                                                      if m['pool_id'] not in
                                                      removed_ids]
 
-            kubernetes = clients.get_kubernetes_client()
-            try:
-                kubernetes.patch_crd('status', loadbalancer_crd[
-                    'metadata']['selfLink'], loadbalancer_crd[
-                        'status'])
-            except k_exc.K8sResourceNotFound:
-                LOG.debug('KuryrLoadbalancer CRD not found %s',
-                          loadbalancer_crd)
-            except k_exc.K8sClientException:
-                LOG.exception('Error updating KuryLoadbalancer CRD %s',
-                              loadbalancer_crd)
-                raise
+            if not self._patch_status(loadbalancer_crd):
+                return False
         return bool(removed_ids)
 
     def _sync_lbaas_listeners(self, loadbalancer_crd):
@@ -570,6 +604,8 @@ class KuryrLoadBalancerHandler(k8s_base.ResourceEventHandler):
     def _add_new_listeners(self, loadbalancer_crd):
         changed = False
         lb_crd_spec_ports = loadbalancer_crd['spec'].get('ports')
+        spec_t_cli = loadbalancer_crd['spec'].get('timeout_client_data', 0)
+        spec_t_mb = loadbalancer_crd['spec'].get('timeout_member_data', 0)
         if not lb_crd_spec_ports:
             return changed
         lbaas_spec_ports = sorted(lb_crd_spec_ports,
@@ -580,9 +616,13 @@ class KuryrLoadBalancerHandler(k8s_base.ResourceEventHandler):
             name = "%s:%s" % (loadbalancer_crd['status']['loadbalancer'][
                 'name'], protocol)
 
-            listener = [l for l in loadbalancer_crd['status'].get(
-                'listeners', []) if l['port'] == port and l[
-                    'protocol'] == protocol]
+            listener = []
+            for l in loadbalancer_crd['status'].get('listeners', []):
+                timeout_cli = l.get('timeout_client_data', 0)
+                timeout_mb = l.get('timeout_member_data', 0)
+                if l['port'] == port and l['protocol'] == protocol:
+                    if timeout_cli == spec_t_cli and timeout_mb == spec_t_mb:
+                        listener.append(l)
 
             if listener:
                 continue
@@ -597,31 +637,32 @@ class KuryrLoadBalancerHandler(k8s_base.ResourceEventHandler):
                 LOG.warning("Skipping listener creation for %s as another one"
                             " already exists with port %s", name, port)
                 continue
+            if protocol == "SCTP" and not self._drv_lbaas.sctp_supported():
+                LOG.warning("Skipping listener creation as provider does"
+                            " not support %s protocol", protocol)
+                continue
             listener = self._drv_lbaas.ensure_listener(
                 loadbalancer=loadbalancer_crd['status'].get('loadbalancer'),
                 protocol=protocol,
                 port=port,
-                service_type=loadbalancer_crd['spec'].get('type'))
+                service_type=loadbalancer_crd['spec'].get('type'),
+                timeout_client_data=spec_t_cli,
+                timeout_member_data=spec_t_mb)
+
             if listener is not None:
                 listeners = loadbalancer_crd['status'].get('listeners', [])
                 if listeners:
+                    for pre_listener in listeners:
+                        if pre_listener['id'] == listener['id']:
+                            listeners.remove(pre_listener)
                     listeners.append(listener)
                 else:
                     loadbalancer_crd['status']['listeners'] = []
                     loadbalancer_crd['status'].get('listeners', []).append(
                         listener)
 
-                kubernetes = clients.get_kubernetes_client()
-                try:
-                    kubernetes.patch_crd('status', loadbalancer_crd[
-                        'metadata']['selfLink'], loadbalancer_crd['status'])
-                except k_exc.K8sResourceNotFound:
-                    LOG.debug('KuryrLoadbalancer CRD not found %s',
-                              loadbalancer_crd)
-                except k_exc.K8sClientException:
-                    LOG.exception('Error updating KuryrLoadbalancer CRD %s',
-                                  loadbalancer_crd)
-                    raise
+                if not self._patch_status(loadbalancer_crd):
+                    return False
                 changed = True
         return changed
 
@@ -641,18 +682,8 @@ class KuryrLoadBalancerHandler(k8s_base.ResourceEventHandler):
                                                           []) if l['id']
                 not in removed_ids]
 
-            kubernetes = clients.get_kubernetes_client()
-            try:
-                kubernetes.patch_crd('status', loadbalancer_crd[
-                    'metadata']['selfLink'], loadbalancer_crd[
-                        'status'])
-            except k_exc.K8sResourceNotFound:
-                LOG.debug('KuryrLoadbalancer CRD not found %s',
-                          loadbalancer_crd)
-            except k_exc.K8sClientException:
-                LOG.exception('Error updating KuryLoadbalancer CRD %s',
-                              loadbalancer_crd)
-                raise
+            if not self._patch_status(loadbalancer_crd):
+                return False
         return bool(removed_ids)
 
     def _update_lb_status(self, lb_crd):
@@ -676,7 +707,6 @@ class KuryrLoadBalancerHandler(k8s_base.ResourceEventHandler):
             raise
 
     def _sync_lbaas_loadbalancer(self, loadbalancer_crd):
-        changed = False
         lb = loadbalancer_crd['status'].get('loadbalancer')
 
         if lb and lb['ip'] != loadbalancer_crd['spec'].get('ip'):
@@ -715,20 +745,8 @@ class KuryrLoadBalancerHandler(k8s_base.ResourceEventHandler):
                     provider=loadbalancer_crd['spec'].get('provider'))
                 loadbalancer_crd['status']['loadbalancer'] = lb
 
-            kubernetes = clients.get_kubernetes_client()
-            try:
-                kubernetes.patch_crd('status', loadbalancer_crd['metadata'][
-                    'selfLink'], loadbalancer_crd['status'])
-            except k_exc.K8sResourceNotFound:
-                LOG.debug('KuryrLoadbalancer CRD not found %s',
-                          loadbalancer_crd)
-            except k_exc.K8sClientException:
-                LOG.exception('Error updating KuryrLoadbalancer CRD %s',
-                              loadbalancer_crd)
-                raise
-            changed = True
-
-        return changed
+            return self._patch_status(loadbalancer_crd)
+        return False
 
     def _ensure_release_lbaas(self, loadbalancer_crd):
         attempts = 0
@@ -757,17 +775,7 @@ class KuryrLoadBalancerHandler(k8s_base.ResourceEventHandler):
                 retry = True
 
             loadbalancer_crd['status'] = {}
-            k8s = clients.get_kubernetes_client()
-            try:
-                k8s.patch_crd('status', loadbalancer_crd['metadata'][
-                    'selfLink'], loadbalancer_crd['status'])
-            except k_exc.K8sResourceNotFound:
-                LOG.debug('KuryrLoadbalancer CRD not found %s',
-                          loadbalancer_crd)
-            except k_exc.K8sClientException:
-                LOG.exception('Error updating KuryrLoadbalancer CRD %s',
-                              loadbalancer_crd)
-                raise
+            self._patch_status(loadbalancer_crd)
             # NOTE(ltomasbo): give some extra time to ensure the Load
             # Balancer VIP is also released
             time.sleep(1)
