@@ -23,10 +23,12 @@ from openstack.load_balancer.v2 import member as o_mem
 from openstack.load_balancer.v2 import pool as o_pool
 from oslo_config import cfg
 
+from kuryr_kubernetes import constants as k_const
 from kuryr_kubernetes.controller.drivers import lbaasv2 as d_lbaasv2
 from kuryr_kubernetes import exceptions as k_exc
 from kuryr_kubernetes.objects import lbaas as obj_lbaas
 from kuryr_kubernetes.tests import base as test_base
+from kuryr_kubernetes.tests import fake
 from kuryr_kubernetes.tests.unit import kuryr_fixtures as k_fix
 
 CONF = cfg.CONF
@@ -55,6 +57,20 @@ OCTAVIA_VERSIONS = {
     },
 }
 
+BAD_OCTAVIA_VERSIONS = {
+    'regionOne': {
+        'public': {
+            'load-balancer': [
+                {
+                    'status': 'CURRENT',
+                    'version': None,
+                    'raw_status': u'CURRENT',
+                },
+            ],
+        },
+    },
+}
+
 
 class TestLBaaSv2Driver(test_base.TestCase):
     @mock.patch('kuryr_kubernetes.controller.drivers.lbaasv2.LBaaSv2Driver.'
@@ -108,6 +124,12 @@ class TestLBaaSv2Driver(test_base.TestCase):
         self.assertEqual((2, 2),
                          d_lbaasv2.LBaaSv2Driver.get_octavia_version(None))
 
+    def test_get_octavia_version_is_none(self):
+        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
+        lbaas.get_all_version_data.return_value = BAD_OCTAVIA_VERSIONS
+        self.assertRaises(k_exc.UnreachableOctavia,
+                          d_lbaasv2.LBaaSv2Driver.get_octavia_version, None)
+
     def test_ensure_loadbalancer(self):
         os_net = self.useFixture(k_fix.MockNetworkClient()).client
         cls = d_lbaasv2.LBaaSv2Driver
@@ -123,14 +145,13 @@ class TestLBaaSv2Driver(test_base.TestCase):
         sg_ids = ['foo', 'bar']
         lb_name = 'just_a_name'
 
-        m_driver._ensure.return_value = expected_resp
+        m_driver._ensure_loadbalancer.return_value = expected_resp
         os_net.update_port = mock.Mock()
         resp = cls.ensure_loadbalancer(m_driver, lb_name, project_id,
                                        subnet_id, ip, sg_ids, 'ClusterIP')
-        m_driver._ensure.assert_called_once_with(m_driver._create_loadbalancer,
-                                                 m_driver._find_loadbalancer,
-                                                 mock.ANY)
-        req = m_driver._ensure.call_args[0][2]
+        m_driver._ensure_loadbalancer.assert_called_once_with(
+            mock.ANY)
+        req = m_driver._ensure_loadbalancer.call_args[0][0]
         self.assertEqual(lb_name, req['name'])
         self.assertEqual(project_id, req['project_id'])
         self.assertEqual(subnet_id, req['subnet_id'])
@@ -148,7 +169,7 @@ class TestLBaaSv2Driver(test_base.TestCase):
         # TODO(ivc): handle security groups
         sg_ids = []
 
-        m_driver._ensure.return_value = None
+        m_driver._ensure_loadbalancer.return_value = None
         self.assertRaises(k_exc.ResourceNotReady, cls.ensure_loadbalancer,
                           m_driver, name, project_id, subnet_id, ip,
                           sg_ids, 'ClusterIP')
@@ -194,8 +215,7 @@ class TestLBaaSv2Driver(test_base.TestCase):
         # TODO(ivc): handle security groups
         m_driver._ensure_provisioned.return_value = expected_resp
 
-        resp = cls.ensure_listener(m_driver, loadbalancer,
-                                   protocol, port)
+        resp = cls.ensure_listener(m_driver, loadbalancer, protocol, port)
 
         m_driver._ensure_provisioned.assert_called_once_with(
             loadbalancer, mock.ANY, m_driver._create_listener,
@@ -309,6 +329,7 @@ class TestLBaaSv2Driver(test_base.TestCase):
                                                   pool['id'])
 
     def test_ensure_member(self):
+        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
         cls = d_lbaasv2.LBaaSv2Driver
         m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
         expected_resp = mock.sentinel.expected_resp
@@ -335,7 +356,7 @@ class TestLBaaSv2Driver(test_base.TestCase):
 
         m_driver._ensure_provisioned.assert_called_once_with(
             loadbalancer, mock.ANY, m_driver._create_member,
-            m_driver._find_member)
+            m_driver._find_member, update=lbaas.update_member)
         member = m_driver._ensure_provisioned.call_args[0][1]
         self.assertEqual("%s/%s:%s" % (namespace, name, port), member['name'])
         self.assertEqual(pool['project_id'], member['project_id'])
@@ -576,6 +597,36 @@ class TestLBaaSv2Driver(test_base.TestCase):
         self.assertEqual(listener, ret)
         self.assertEqual(listener_id, ret['id'])
 
+    def test_create_listener_with_different_timeouts(self):
+        cls = d_lbaasv2.LBaaSv2Driver
+        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
+        listener = {
+            'name': 'TEST_NAME',
+            'project_id': 'TEST_PROJECT',
+            'loadbalancer_id': '00EE9E11-91C2-41CF-8FD4-7970579E5C4C',
+            'protocol': 'TCP',
+            'port': 5678,
+            'timeout_client_data': 75000,
+            'timeout_member_data': 0
+        }
+        listener_id = 'A57B7771-6050-4CA8-A63C-443493EC98AB'
+
+        req = {
+            'name': listener['name'],
+            'project_id': listener['project_id'],
+            'loadbalancer_id': listener['loadbalancer_id'],
+            'protocol': listener['protocol'],
+            'protocol_port': listener['port'],
+            'timeout_client_data': listener['timeout_client_data']}
+        resp = o_lis.Listener(id=listener_id)
+        lbaas.create_listener.return_value = resp
+
+        ret = cls._create_listener(m_driver, listener)
+        lbaas.create_listener.assert_called_once_with(**req)
+        self.assertEqual(listener, ret)
+        self.assertEqual(listener_id, ret['id'])
+
     def test_find_listener(self):
         lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
         cls = d_lbaasv2.LBaaSv2Driver
@@ -809,17 +860,17 @@ class TestLBaaSv2Driver(test_base.TestCase):
             'port': 1234
         }
         member_id = '3A70CEC0-392D-4BC1-A27C-06E63A0FD54F'
-        resp = iter([o_mem.Member(id=member_id)])
+        resp = iter([o_mem.Member(id=member_id, name='TEST_NAME')])
         lbaas.members.return_value = resp
-
         ret = cls._find_member(m_driver, member, loadbalancer)
         lbaas.members.assert_called_once_with(
             member['pool_id'],
-            name=member['name'],
             project_id=member['project_id'],
             subnet_id=member['subnet_id'],
             address=member['ip'],
             protocol_port=member['port'])
+        # the member dict is copied, so the id is added to the return obj
+        member['id'] = member_id
         self.assertEqual(member, ret)
         self.assertEqual(member_id, ret['id'])
 
@@ -842,7 +893,6 @@ class TestLBaaSv2Driver(test_base.TestCase):
         ret = cls._find_member(m_driver, member, loadbalancer)
         lbaas.members.assert_called_once_with(
             member['pool_id'],
-            name=member['name'],
             project_id=member['project_id'],
             subnet_id=member['subnet_id'],
             address=member['ip'],
@@ -853,12 +903,14 @@ class TestLBaaSv2Driver(test_base.TestCase):
         cls = d_lbaasv2.LBaaSv2Driver
         m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
         obj = mock.Mock()
+        lb = mock.Mock()
         m_create = mock.Mock()
         m_find = mock.Mock()
         expected_result = mock.sentinel.expected_result
         m_create.return_value = expected_result
 
-        ret = cls._ensure(m_driver, m_create, m_find, obj)
+        ret = cls._ensure(m_driver, m_create, m_find,
+                          obj, lb)
         m_create.assert_called_once_with(obj)
         self.assertEqual(expected_result, ret)
 
@@ -866,15 +918,17 @@ class TestLBaaSv2Driver(test_base.TestCase):
         cls = d_lbaasv2.LBaaSv2Driver
         m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
         obj = mock.Mock()
+        lb = mock.Mock()
         m_create = mock.Mock()
         m_find = mock.Mock()
-        expected_result = mock.sentinel.expected_result
+        expected_result = None
         m_create.side_effect = exception_value
         m_find.return_value = expected_result
 
-        ret = cls._ensure(m_driver, m_create, m_find, obj)
+        ret = cls._ensure(m_driver, m_create, m_find,
+                          obj, lb)
         m_create.assert_called_once_with(obj)
-        m_find.assert_called_once_with(obj)
+        m_find.assert_called_once_with(obj, lb)
         self.assertEqual(expected_result, ret)
 
     def test_ensure_with_conflict(self):
@@ -1038,3 +1092,505 @@ class TestLBaaSv2Driver(test_base.TestCase):
     def test_provisioning_timer(self):
         # REVISIT(ivc): add test if _provisioning_timer is to stay
         self.skipTest("not implemented")
+
+
+class TestLBaaSv2AppyMembersSecurityGroup(test_base.TestCase):
+
+    def setUp(self):
+        super().setUp()
+        self.lb = {'id': 'a4de5f1a-ac03-45b1-951d-39f108d52e7d',
+                   'ip': '10.0.0.142',
+                   'name': 'default/lb',
+                   'port_id': '5be1b3c4-7d44-4597-9294-cadafdf1ec69',
+                   'project_id': '7ef23242bb3f4773a58da681421ab26e',
+                   'provider': 'amphora',
+                   'security_groups': ['328900a2-c328-41cc-946f-56ae8720ec0d'],
+                   'subnet_id': 'c85e2e10-1fad-4218-ad10-7de4aa5de7ce'}
+        self.port = 80
+        self.target_port = 8080
+        self.protocol = 'TCP'
+        self.sg_rule_name = 'default/lb:TCP:80'
+        self.listener_id = '858869ec-e4fa-4715-b22f-bd08889c6235'
+        self.new_sgs = ['48cfc812-a442-44bf-989f-8dbaf23a7007']
+        self.vip = fake.get_port_obj()
+
+    @mock.patch('kuryr_kubernetes.clients.get_network_client')
+    def test__apply_members_security_groups_no_enforce(self, gnc):
+        CONF.set_override('enforce_sg_rules', False, group='octavia_defaults')
+        self.addCleanup(CONF.clear_override, 'enforce_sg_rules',
+                        group='octavia_defaults')
+        cls = d_lbaasv2.LBaaSv2Driver
+        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        m_driver._get_vip_port.return_value = None
+
+        cls._apply_members_security_groups(m_driver, self.lb, self.port,
+                                           self.target_port, self.protocol,
+                                           self.sg_rule_name,
+                                           self.listener_id, self.new_sgs)
+
+        m_driver._get_vip_port.assert_not_called()
+
+    @mock.patch('kuryr_kubernetes.clients.get_network_client')
+    def test__apply_members_security_groups_no_vip(self, gnc):
+        cls = d_lbaasv2.LBaaSv2Driver
+        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        m_driver._get_vip_port.return_value = None
+
+        cls._apply_members_security_groups(m_driver, self.lb, self.port,
+                                           self.target_port, self.protocol,
+                                           self.sg_rule_name,
+                                           self.listener_id, self.new_sgs)
+
+        m_driver._get_vip_port.assert_called_once_with(self.lb)
+
+    @mock.patch('kuryr_kubernetes.clients.get_network_client')
+    def test__apply_members_security_groups_no_sg(self, gnc):
+        self.new_sgs = None
+        self.vip.security_group_ids = []
+        cls = d_lbaasv2.LBaaSv2Driver
+        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        m_driver._get_vip_port.return_value = self.vip
+
+        self.assertRaises(k_exc.ResourceNotReady,
+                          cls._apply_members_security_groups, m_driver,
+                          self.lb, self.port, self.target_port, self.protocol,
+                          self.sg_rule_name, self.listener_id, self.new_sgs)
+
+        m_driver._get_vip_port.assert_called_once_with(self.lb)
+
+    @mock.patch('kuryr_kubernetes.clients.get_network_client')
+    def test__apply_members_security_groups_conf_with_octavia_acls(self, gnc):
+        self.new_sgs = None
+        cls = d_lbaasv2.LBaaSv2Driver
+        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        m_driver._get_vip_port = mock.Mock(return_value=self.vip)
+        m_driver._octavia_acls = True
+        m_driver._create_listeners_acls = mock.Mock()
+
+        cls._apply_members_security_groups(m_driver, self.lb, self.port,
+                                           self.target_port, self.protocol,
+                                           self.sg_rule_name, self.listener_id,
+                                           self.new_sgs)
+
+        m_driver._get_vip_port.assert_called_once_with(self.lb)
+        m_driver._create_listeners_acls.assert_called_once_with(
+            self.lb, self.port, self.target_port, self.protocol,
+            self.vip.security_group_ids[0], self.new_sgs, self.listener_id)
+
+    def test__apply_members_security_groups_new_sgs(self):
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
+        cls = d_lbaasv2.LBaaSv2Driver
+        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        m_driver._get_vip_port.return_value = self.vip
+        m_driver._octavia_acls = False
+        os_net.security_group_rules.return_value = []
+        CONF.set_override('pod_security_groups', [], group='neutron_defaults')
+        self.addCleanup(CONF.clear_override, 'pod_security_groups',
+                        group='neutron_defaults')
+
+        cls._apply_members_security_groups(m_driver, self.lb, self.port,
+                                           self.target_port, self.protocol,
+                                           self.sg_rule_name, self.listener_id,
+                                           self.new_sgs)
+
+        m_driver._get_vip_port.assert_called_once_with(self.lb)
+        os_net.security_group_rules.assert_has_calls([
+            mock.call(security_group_id=self.vip.security_group_ids[0],
+                      project_id=self.lb['project_id']),
+            mock.call(security_group_id=self.new_sgs[0])])
+
+    def test__apply_members_security_groups_conf_lb_sgs(self):
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
+        cls = d_lbaasv2.LBaaSv2Driver
+        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        m_driver._get_vip_port.return_value = self.vip
+        m_driver._octavia_acls = False
+        sgr = fake.get_sgr_obj()
+        os_net.security_group_rules.side_effect = ([], [sgr])
+        self.new_sgs = []
+        CONF.set_override('pod_security_groups', [], group='neutron_defaults')
+        self.addCleanup(CONF.clear_override, 'pod_security_groups',
+                        group='neutron_defaults')
+
+        cls._apply_members_security_groups(m_driver, self.lb, self.port,
+                                           self.target_port, self.protocol,
+                                           self.sg_rule_name,
+                                           self.listener_id, self.new_sgs)
+
+        m_driver._get_vip_port.assert_called_once_with(self.lb)
+        os_net.security_group_rules.assert_has_calls([
+            mock.call(security_group_id=self.vip.security_group_ids[0],
+                      project_id=self.lb['project_id']),
+            mock.call(security_group_id=self.lb['security_groups'][0])])
+        os_net.create_security_group_rule.assert_called_once_with(
+            direction='ingress',
+            ether_type=k_const.IPv4,
+            port_range_min=self.port,
+            port_range_max=self.port,
+            protocol=self.protocol,
+            remote_ip_prefix=sgr.remote_ip_prefix,
+            security_group_id=sgr.security_group_id,
+            description=self.sg_rule_name)
+
+    def test__apply_members_security_groups_conf_lb_sgs_conflict(self):
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
+        cls = d_lbaasv2.LBaaSv2Driver
+        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        m_driver._get_vip_port.return_value = self.vip
+        m_driver._octavia_acls = False
+        sgr = fake.get_sgr_obj()
+        os_net.security_group_rules.side_effect = ([], [sgr])
+        os_net.create_security_group_rule.side_effect = (os_exc
+                                                         .ConflictException)
+        self.new_sgs = []
+        CONF.set_override('pod_security_groups', [], group='neutron_defaults')
+        self.addCleanup(CONF.clear_override, 'pod_security_groups',
+                        group='neutron_defaults')
+
+        cls._apply_members_security_groups(m_driver, self.lb, self.port,
+                                           self.target_port, self.protocol,
+                                           self.sg_rule_name,
+                                           self.listener_id, self.new_sgs)
+
+        m_driver._get_vip_port.assert_called_once_with(self.lb)
+        os_net.security_group_rules.assert_has_calls([
+            mock.call(security_group_id=self.vip.security_group_ids[0],
+                      project_id=self.lb['project_id']),
+            mock.call(security_group_id=self.lb['security_groups'][0])])
+        os_net.create_security_group_rule.assert_called_once_with(
+            direction='ingress',
+            ether_type=k_const.IPv4,
+            port_range_min=self.port,
+            port_range_max=self.port,
+            protocol=self.protocol,
+            remote_ip_prefix=None,
+            security_group_id=self.vip.security_group_ids[0],
+            description=self.sg_rule_name)
+
+    def test__apply_members_security_groups_conf_lb_sgs_sdkexception(self):
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
+        cls = d_lbaasv2.LBaaSv2Driver
+        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        m_driver._get_vip_port.return_value = self.vip
+        m_driver._octavia_acls = False
+        sgr = fake.get_sgr_obj()
+        os_net.security_group_rules.side_effect = ([], [sgr])
+        os_net.create_security_group_rule.side_effect = os_exc.SDKException
+        self.new_sgs = []
+        CONF.set_override('pod_security_groups', [], group='neutron_defaults')
+        self.addCleanup(CONF.clear_override, 'pod_security_groups',
+                        group='neutron_defaults')
+
+        cls._apply_members_security_groups(m_driver, self.lb, self.port,
+                                           self.target_port, self.protocol,
+                                           self.sg_rule_name,
+                                           self.listener_id, self.new_sgs)
+
+        m_driver._get_vip_port.assert_called_once_with(self.lb)
+        os_net.security_group_rules.assert_has_calls([
+            mock.call(security_group_id=self.vip.security_group_ids[0],
+                      project_id=self.lb['project_id']),
+            mock.call(security_group_id=self.lb['security_groups'][0])])
+        os_net.create_security_group_rule.assert_called_once_with(
+            direction='ingress',
+            ether_type=k_const.IPv4,
+            port_range_min=self.port,
+            port_range_max=self.port,
+            protocol=self.protocol,
+            remote_ip_prefix=None,
+            security_group_id=self.vip.security_group_ids[0],
+            description=self.sg_rule_name)
+
+    @mock.patch("kuryr_kubernetes.utils.get_service_subnet_version",
+                return_value=k_const.IP_VERSION_6)
+    def test__apply_members_security_groups_ipv6_add_default(self, gssv):
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
+        cls = d_lbaasv2.LBaaSv2Driver
+        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        m_driver._get_vip_port.return_value = self.vip
+        m_driver._octavia_acls = False
+        os_net.security_group_rules.return_value = []
+        CONF.set_override('pod_security_groups', self.new_sgs,
+                          group='neutron_defaults')
+        self.addCleanup(CONF.clear_override, 'pod_security_groups',
+                        group='neutron_defaults')
+
+        cls._apply_members_security_groups(m_driver, self.lb, self.port,
+                                           self.target_port, self.protocol,
+                                           self.sg_rule_name, self.listener_id,
+                                           self.new_sgs)
+
+        m_driver._get_vip_port.assert_called_once_with(self.lb)
+        os_net.security_group_rules.assert_called_once_with(
+            security_group_id=self.vip.security_group_ids[0],
+            project_id=self.lb['project_id'])
+        os_net.create_security_group_rule.assert_called_once_with(
+            direction='ingress',
+            ether_type=k_const.IPv6,
+            port_range_min=self.port,
+            port_range_max=self.port,
+            protocol=self.protocol,
+            security_group_id=self.vip.security_group_ids[0],
+            description=self.sg_rule_name)
+
+    def test__apply_members_security_groups_add_default_conflict(self):
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
+        cls = d_lbaasv2.LBaaSv2Driver
+        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        m_driver._get_vip_port.return_value = self.vip
+        m_driver._octavia_acls = False
+        os_net.security_group_rules.return_value = []
+        CONF.set_override('pod_security_groups', self.new_sgs,
+                          group='neutron_defaults')
+        self.addCleanup(CONF.clear_override, 'pod_security_groups',
+                        group='neutron_defaults')
+        os_net.create_security_group_rule.side_effect = (os_exc
+                                                         .ConflictException)
+
+        cls._apply_members_security_groups(m_driver, self.lb, self.port,
+                                           self.target_port, self.protocol,
+                                           self.sg_rule_name, self.listener_id,
+                                           self.new_sgs)
+
+        m_driver._get_vip_port.assert_called_once_with(self.lb)
+        os_net.security_group_rules.assert_called_once_with(
+            security_group_id=self.vip.security_group_ids[0],
+            project_id=self.lb['project_id'])
+        os_net.create_security_group_rule.assert_called_once_with(
+            direction='ingress',
+            ether_type=k_const.IPv4,
+            port_range_min=self.port,
+            port_range_max=self.port,
+            protocol=self.protocol,
+            security_group_id=self.vip.security_group_ids[0],
+            description=self.sg_rule_name)
+
+    def test__apply_members_security_groups_add_default_sdk_exception(self):
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
+        cls = d_lbaasv2.LBaaSv2Driver
+        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        m_driver._get_vip_port.return_value = self.vip
+        m_driver._octavia_acls = False
+        os_net.security_group_rules.return_value = []
+        CONF.set_override('pod_security_groups', self.new_sgs,
+                          group='neutron_defaults')
+        self.addCleanup(CONF.clear_override, 'pod_security_groups',
+                        group='neutron_defaults')
+        os_net.create_security_group_rule.side_effect = os_exc.SDKException
+
+        cls._apply_members_security_groups(m_driver, self.lb, self.port,
+                                           self.target_port, self.protocol,
+                                           self.sg_rule_name, self.listener_id,
+                                           self.new_sgs)
+
+        m_driver._get_vip_port.assert_called_once_with(self.lb)
+        os_net.security_group_rules.assert_called_once_with(
+            security_group_id=self.vip.security_group_ids[0],
+            project_id=self.lb['project_id'])
+        os_net.create_security_group_rule.assert_called_once_with(
+            direction='ingress',
+            ether_type=k_const.IPv4,
+            port_range_min=self.port,
+            port_range_max=self.port,
+            protocol=self.protocol,
+            security_group_id=self.vip.security_group_ids[0],
+            description=self.sg_rule_name)
+
+    def test__apply_members_security_groups_same_sg(self):
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
+        cls = d_lbaasv2.LBaaSv2Driver
+        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        self.vip.security_group_ids = self.new_sgs
+        m_driver._get_vip_port.return_value = self.vip
+        m_driver._octavia_acls = False
+        os_net.security_group_rules.return_value = []
+        CONF.set_override('pod_security_groups', [], group='neutron_defaults')
+        self.addCleanup(CONF.clear_override, 'pod_security_groups',
+                        group='neutron_defaults')
+
+        cls._apply_members_security_groups(m_driver, self.lb, self.port,
+                                           self.target_port, self.protocol,
+                                           self.sg_rule_name, self.listener_id,
+                                           self.new_sgs)
+
+        m_driver._get_vip_port.assert_called_once_with(self.lb)
+        os_net.security_group_rules.assert_called_once_with(
+            security_group_id=self.vip.security_group_ids[0],
+            project_id=self.lb['project_id'])
+
+    def test__apply_members_security_groups_unmatched_target_port(self):
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
+        cls = d_lbaasv2.LBaaSv2Driver
+        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        m_driver._get_vip_port.return_value = self.vip
+        m_driver._octavia_acls = False
+        sgr = fake.get_sgr_obj()
+        self.target_port = 9090
+        os_net.security_group_rules.side_effect = ([], [sgr])
+        self.new_sgs = []
+        CONF.set_override('pod_security_groups', [], group='neutron_defaults')
+        self.addCleanup(CONF.clear_override, 'pod_security_groups',
+                        group='neutron_defaults')
+
+        cls._apply_members_security_groups(m_driver, self.lb, self.port,
+                                           self.target_port, self.protocol,
+                                           self.sg_rule_name,
+                                           self.listener_id, self.new_sgs)
+
+        m_driver._get_vip_port.assert_called_once_with(self.lb)
+        os_net.security_group_rules.assert_has_calls([
+            mock.call(security_group_id=self.vip.security_group_ids[0],
+                      project_id=self.lb['project_id']),
+            mock.call(security_group_id=self.lb['security_groups'][0])])
+        os_net.create_security_group_rule.assert_not_called()
+
+    def test__apply_members_security_groups_egress(self):
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
+        cls = d_lbaasv2.LBaaSv2Driver
+        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        m_driver._get_vip_port.return_value = self.vip
+        m_driver._octavia_acls = False
+        sgr = fake.get_sgr_obj(direction='egress')
+        os_net.security_group_rules.side_effect = ([], [sgr])
+        self.new_sgs = []
+        CONF.set_override('pod_security_groups', [], group='neutron_defaults')
+        self.addCleanup(CONF.clear_override, 'pod_security_groups',
+                        group='neutron_defaults')
+
+        cls._apply_members_security_groups(m_driver, self.lb, self.port,
+                                           self.target_port, self.protocol,
+                                           self.sg_rule_name,
+                                           self.listener_id, self.new_sgs)
+
+        m_driver._get_vip_port.assert_called_once_with(self.lb)
+        os_net.security_group_rules.assert_has_calls([
+            mock.call(security_group_id=self.vip.security_group_ids[0],
+                      project_id=self.lb['project_id']),
+            mock.call(security_group_id=self.lb['security_groups'][0])])
+        os_net.create_security_group_rule.assert_not_called()
+
+    def test__apply_members_security_groups_no_delete_lbaas_rules(self):
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
+        cls = d_lbaasv2.LBaaSv2Driver
+        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        m_driver._get_vip_port.return_value = self.vip
+        m_driver._octavia_acls = False
+        self.lb['security_groups'] = []
+        self.new_sgs = []
+        sgr = fake.get_sgr_obj()
+        os_net.security_group_rules.return_value = [sgr]
+
+        cls._apply_members_security_groups(m_driver, self.lb, self.port,
+                                           self.target_port, self.protocol,
+                                           self.sg_rule_name,
+                                           self.listener_id, self.new_sgs)
+
+        m_driver._get_vip_port.assert_called_once_with(self.lb)
+        os_net.security_group_rules.assert_called_once_with(
+            security_group_id=self.vip.security_group_ids[0],
+            project_id=self.lb['project_id'])
+        os_net.create_security_group_rule.assert_not_called()
+
+    def test__apply_members_security_groups_delete_matched_lbaas_rules(self):
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
+        cls = d_lbaasv2.LBaaSv2Driver
+        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        m_driver._get_vip_port.return_value = self.vip
+        m_driver._octavia_acls = False
+        sgr = fake.get_sgr_obj()
+        os_net.security_group_rules.side_effect = ([sgr], [sgr])
+        self.new_sgs = []
+        CONF.set_override('pod_security_groups', [], group='neutron_defaults')
+        self.addCleanup(CONF.clear_override, 'pod_security_groups',
+                        group='neutron_defaults')
+
+        cls._apply_members_security_groups(m_driver, self.lb, self.port,
+                                           self.target_port, self.protocol,
+                                           self.sg_rule_name,
+                                           self.listener_id, self.new_sgs)
+
+        m_driver._get_vip_port.assert_called_once_with(self.lb)
+        os_net.security_group_rules.assert_has_calls([
+            mock.call(security_group_id=self.vip.security_group_ids[0],
+                      project_id=self.lb['project_id']),
+            mock.call(security_group_id=self.lb['security_groups'][0])])
+        os_net.create_security_group_rule.assert_called_once_with(
+            direction='ingress',
+            ether_type=k_const.IPv4,
+            port_range_min=self.port,
+            port_range_max=self.port,
+            protocol=self.protocol,
+            remote_ip_prefix=sgr.remote_ip_prefix,
+            security_group_id=sgr.security_group_id,
+            description=self.sg_rule_name)
+        os_net.delete_security_group_rule.assert_called_once_with(sgr.id)
+
+    def test__apply_members_security_groups_delete_unmatched_lbaas_rules(self):
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
+        cls = d_lbaasv2.LBaaSv2Driver
+        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        m_driver._get_vip_port.return_value = self.vip
+        m_driver._octavia_acls = False
+        sgr = fake.get_sgr_obj()
+        os_net.security_group_rules.side_effect = ([sgr], [sgr])
+        self.new_sgs = []
+        CONF.set_override('pod_security_groups', [], group='neutron_defaults')
+        self.addCleanup(CONF.clear_override, 'pod_security_groups',
+                        group='neutron_defaults')
+        self.port = 8080
+
+        cls._apply_members_security_groups(m_driver, self.lb, self.port,
+                                           self.target_port, self.protocol,
+                                           self.sg_rule_name,
+                                           self.listener_id, self.new_sgs)
+
+        m_driver._get_vip_port.assert_called_once_with(self.lb)
+        os_net.security_group_rules.assert_has_calls([
+            mock.call(security_group_id=self.vip.security_group_ids[0],
+                      project_id=self.lb['project_id']),
+            mock.call(security_group_id=self.lb['security_groups'][0])])
+        os_net.create_security_group_rule.assert_called_once_with(
+            direction='ingress',
+            ether_type=k_const.IPv4,
+            port_range_min=self.port,
+            port_range_max=self.port,
+            protocol=self.protocol,
+            remote_ip_prefix=sgr.remote_ip_prefix,
+            security_group_id=sgr.security_group_id,
+            description=self.sg_rule_name)
+        m_driver._delete_rule_if_no_match.assert_called_once_with(sgr, [sgr])
+
+    def test__apply_members_security_groups_delete_no_default_lbaas_rules(
+            self):
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
+        cls = d_lbaasv2.LBaaSv2Driver
+        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        m_driver._get_vip_port.return_value = self.vip
+        m_driver._octavia_acls = False
+        sgr = fake.get_sgr_obj()
+        os_net.security_group_rules.side_effect = ([sgr], [sgr])
+        self.new_sgs = []
+        CONF.set_override('pod_security_groups', [], group='neutron_defaults')
+        self.addCleanup(CONF.clear_override, 'pod_security_groups',
+                        group='neutron_defaults')
+        m_driver._is_default_rule.return_value = False
+
+        cls._apply_members_security_groups(m_driver, self.lb, self.port,
+                                           self.target_port, self.protocol,
+                                           self.sg_rule_name,
+                                           self.listener_id, self.new_sgs)
+
+        m_driver._get_vip_port.assert_called_once_with(self.lb)
+        os_net.security_group_rules.assert_has_calls([
+            mock.call(security_group_id=self.vip.security_group_ids[0],
+                      project_id=self.lb['project_id']),
+            mock.call(security_group_id=self.lb['security_groups'][0])])
+        os_net.create_security_group_rule.assert_called_once_with(
+            direction='ingress',
+            ether_type=k_const.IPv4,
+            port_range_min=self.port,
+            port_range_max=self.port,
+            protocol=self.protocol,
+            remote_ip_prefix=sgr.remote_ip_prefix,
+            security_group_id=sgr.security_group_id,
+            description=self.sg_rule_name)
