@@ -30,6 +30,8 @@ _SUPPORTED_LISTENER_PROT = ('HTTP', 'HTTPS', 'TCP')
 
 def get_lb_crd():
     return {
+            'apiVersion': 'openstack.org/v1',
+            'kind': 'KuryrLoadBalancer',
             "metadata": {
                 "creationTimestamp": "2020-07-28T13:13:30Z",
                 "finalizers": [
@@ -39,7 +41,6 @@ def get_lb_crd():
                 "name": "test",
                 "namespace": "default",
                 "resourceVersion": "111871",
-                "selfLink": "test",
                 "uid": "584fe3ea-04dd-43f7-be2f-713e861694ec"
             },
             "spec": {
@@ -136,6 +137,71 @@ def get_lb_crd():
         }
 
 
+def get_lb_crds():
+    return [
+            {
+             'apiVersion': 'openstack.org/v1',
+             'kind': 'KuryrLoadBalancer',
+             "metadata": {
+                "creationTimestamp": "2020-07-28T13:13:30Z",
+                "finalizers": [
+                    ""
+                ],
+                "generation": 6,
+                "name": "test",
+                "namespace": "default",
+                "resourceVersion": "111871",
+                "uid": "584fe3ea-04dd-43f7-be2f-713e861694ec"
+                },
+             "status": {
+                 "loadbalancer": {
+                     "id": "01234567890",
+                     "ip": "1.2.3.4",
+                     "name": "default/test",
+                     "port_id": "1023456789120",
+                     "project_id": "12345678912",
+                     "provider": "amphora",
+                     "security_groups": [
+                         "1d134e68-5653-4192-bda2-4214319af799",
+                         "31d7b8c2-75f1-4125-9565-8c15c5cf046c"
+                     ],
+                     "subnet_id": "123456789120"
+                    },
+                }
+            },
+            {
+             'apiVersion': 'openstack.org/v1',
+             'kind': 'KuryrLoadBalancer',
+             "metadata": {
+                "creationTimestamp": "2020-07-28T13:13:30Z",
+                "finalizers": [
+                    ""
+                ],
+                "generation": 6,
+                "name": "demo",
+                "namespace": "default",
+                "resourceVersion": "111871",
+                "uid": "584fe3ea-04dd-43f7-be2f-713e861694ec"
+                },
+             "status": {
+                 "loadbalancer": {
+                     "id": "01234567891",
+                     "ip": "1.2.3.4",
+                     "name": "default/demo",
+                     "port_id": "1023456789120",
+                     "project_id": "12345678912",
+                     "provider": "amphora",
+                     "security_groups": [
+                         "1d134e68-5653-4192-bda2-4214319af799",
+                         "31d7b8c2-75f1-4125-9565-8c15c5cf046c"
+                     ],
+                     "subnet_id": "123456789120"
+                    },
+                }
+            }
+        ]
+
+
 class FakeLBaaSDriver(drv_base.LBaaSDriver):
 
     def ensure_loadbalancer(self, name, project_id, subnet_id, ip,
@@ -190,8 +256,10 @@ class FakeLBaaSDriver(drv_base.LBaaSDriver):
         }
 
 
+@mock.patch('kuryr_kubernetes.utils.get_subnets_id_cidrs',
+            mock.Mock(return_value=[('id', 'cidr')]))
 class TestKuryrLoadBalancerHandler(test_base.TestCase):
-
+    @mock.patch('kuryr_kubernetes.utils.get_subnet_cidr')
     @mock.patch('kuryr_kubernetes.controller.drivers.base.'
                 'ServiceProjectDriver.get_instance')
     @mock.patch('kuryr_kubernetes.controller.drivers.base.'
@@ -205,22 +273,30 @@ class TestKuryrLoadBalancerHandler(test_base.TestCase):
                 '.PodProjectDriver.get_instance')
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
                 '.LBaaSDriver.get_instance')
-    def test_init(self, m_get_drv_lbaas, m_get_drv_project,
-                  m_get_drv_subnets, m_get_drv_service_pub_ip, m_cfg,
-                  m_get_svc_sg_drv, m_get_svc_drv_project):
+    @mock.patch('kuryr_kubernetes.controller.drivers.base'
+                '.NodesSubnetsDriver.get_instance')
+    def test_init(self, m_get_drv_node_subnets, m_get_drv_lbaas,
+                  m_get_drv_project, m_get_drv_subnets,
+                  m_get_drv_service_pub_ip, m_cfg, m_get_svc_sg_drv,
+                  m_get_svc_drv_project, m_get_cidr):
         m_get_drv_lbaas.return_value = mock.sentinel.drv_lbaas
         m_get_drv_project.return_value = mock.sentinel.drv_project
         m_get_drv_subnets.return_value = mock.sentinel.drv_subnets
+        m_get_cidr.return_value = '10.0.0.128/26'
         m_get_drv_service_pub_ip.return_value = mock.sentinel.drv_lb_ip
         m_get_svc_drv_project.return_value = mock.sentinel.drv_svc_project
         m_get_svc_sg_drv.return_value = mock.sentinel.drv_sg
+        m_get_drv_node_subnets.return_value = mock.sentinel.drv_node_subnets
         handler = h_lb.KuryrLoadBalancerHandler()
 
         self.assertEqual(mock.sentinel.drv_lbaas, handler._drv_lbaas)
         self.assertEqual(mock.sentinel.drv_project, handler._drv_pod_project)
         self.assertEqual(mock.sentinel.drv_subnets, handler._drv_pod_subnets)
         self.assertEqual(mock.sentinel.drv_lb_ip, handler._drv_service_pub_ip)
+        self.assertEqual(mock.sentinel.drv_node_subnets,
+                         handler._drv_nodes_subnets)
 
+    @mock.patch('kuryr_kubernetes.utils.get_subnet_cidr')
     @mock.patch('kuryr_kubernetes.controller.drivers.base.'
                 'ServiceProjectDriver.get_instance')
     @mock.patch('kuryr_kubernetes.controller.drivers.base.'
@@ -234,22 +310,29 @@ class TestKuryrLoadBalancerHandler(test_base.TestCase):
                 '.PodProjectDriver.get_instance')
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
                 '.LBaaSDriver.get_instance')
-    def test_init_provider_ovn(self, m_get_drv_lbaas, m_get_drv_project,
-                               m_get_drv_subnets, m_get_drv_service_pub_ip,
-                               m_cfg,
-                               m_get_svc_sg_drv, m_get_svc_drv_project):
+    @mock.patch('kuryr_kubernetes.controller.drivers.base'
+                '.NodesSubnetsDriver.get_instance')
+    def test_init_provider_ovn(self, m_get_drv_node_subnets, m_get_drv_lbaas,
+                               m_get_drv_project, m_get_drv_subnets,
+                               m_get_drv_service_pub_ip, m_cfg,
+                               m_get_svc_sg_drv, m_get_svc_drv_project,
+                               m_get_cidr):
+        m_get_cidr.return_value = '10.0.0.128/26'
         m_get_drv_lbaas.return_value = mock.sentinel.drv_lbaas
         m_get_drv_project.return_value = mock.sentinel.drv_project
         m_get_drv_subnets.return_value = mock.sentinel.drv_subnets
         m_get_drv_service_pub_ip.return_value = mock.sentinel.drv_lb_ip
         m_get_svc_drv_project.return_value = mock.sentinel.drv_svc_project
         m_get_svc_sg_drv.return_value = mock.sentinel.drv_sg
+        m_get_drv_node_subnets.return_value = mock.sentinel.drv_node_subnets
         handler = h_lb .KuryrLoadBalancerHandler()
 
         self.assertEqual(mock.sentinel.drv_lbaas, handler._drv_lbaas)
         self.assertEqual(mock.sentinel.drv_project, handler._drv_pod_project)
         self.assertEqual(mock.sentinel.drv_subnets, handler._drv_pod_subnets)
         self.assertEqual(mock.sentinel.drv_lb_ip, handler._drv_service_pub_ip)
+        self.assertEqual(mock.sentinel.drv_node_subnets,
+                         handler._drv_nodes_subnets)
 
     def test_on_present(self):
         m_drv_service_pub_ip = mock.Mock()
@@ -343,32 +426,32 @@ class TestKuryrLoadBalancerHandler(test_base.TestCase):
 
     def test_should_ignore(self):
         m_handler = mock.Mock(spec=h_lb.KuryrLoadBalancerHandler)
-        m_handler._has_pods.return_value = True
         loadbalancer_crd = get_lb_crd()
         loadbalancer_crd['status'] = {}
+        m_handler._has_endpoints.return_value = True
 
         ret = h_lb.KuryrLoadBalancerHandler._should_ignore(
             m_handler, loadbalancer_crd)
         self.assertEqual(False, ret)
 
-        m_handler._has_pods.assert_called_once_with(loadbalancer_crd)
+        m_handler._has_endpoints.assert_called_once_with(loadbalancer_crd)
 
     def test_should_ignore_member_scale_to_0(self):
         m_handler = mock.Mock(spec=h_lb.KuryrLoadBalancerHandler)
-        m_handler._has_pods.return_value = False
+        m_handler._has_endpoints.return_value = False
         loadbalancer_crd = get_lb_crd()
 
         ret = h_lb.KuryrLoadBalancerHandler._should_ignore(
             m_handler, loadbalancer_crd)
         self.assertEqual(False, ret)
 
-        m_handler._has_pods.assert_called_once_with(loadbalancer_crd)
+        m_handler._has_endpoints.assert_called_once_with(loadbalancer_crd)
 
-    def test_has_pods(self):
+    def test_has_endpoints(self):
         crd = get_lb_crd()
         m_handler = mock.Mock(spec=h_lb.KuryrLoadBalancerHandler)
 
-        ret = h_lb.KuryrLoadBalancerHandler._has_pods(m_handler, crd)
+        ret = h_lb.KuryrLoadBalancerHandler._has_endpoints(m_handler, crd)
 
         self.assertEqual(True, ret)
 
@@ -422,6 +505,7 @@ class TestKuryrLoadBalancerHandler(test_base.TestCase):
             for member in crd['status']['members'])
         return observed_targets
 
+    @mock.patch('kuryr_kubernetes.utils.get_subnet_cidr')
     @mock.patch('kuryr_kubernetes.controller.drivers.base.'
                 'ServiceSecurityGroupsDriver.get_instance')
     @mock.patch('kuryr_kubernetes.controller.drivers.base.'
@@ -433,10 +517,13 @@ class TestKuryrLoadBalancerHandler(test_base.TestCase):
                 '.PodProjectDriver.get_instance')
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
                 '.LBaaSDriver.get_instance')
+    @mock.patch('kuryr_kubernetes.controller.drivers.base'
+                '.NodesSubnetsDriver.get_instance', mock.Mock())
     def test_sync_lbaas_members(self, m_get_drv_lbaas, m_get_drv_project,
                                 m_get_drv_subnets, m_k8s, m_svc_project_drv,
-                                m_svc_sg_drv):
+                                m_svc_sg_drv, m_get_cidr):
         # REVISIT(ivc): test methods separately and verify ensure/release
+        m_get_cidr.return_value = '10.0.0.128/26'
         project_id = str(uuid.uuid4())
         subnet_id = str(uuid.uuid4())
         expected_ip = '1.2.3.4'
@@ -453,6 +540,7 @@ class TestKuryrLoadBalancerHandler(test_base.TestCase):
         self.assertEqual(sorted(expected_targets.items()), observed_targets)
         self.assertEqual(expected_ip, str(crd['status']['loadbalancer']['ip']))
 
+    @mock.patch('kuryr_kubernetes.utils.get_subnet_cidr')
     @mock.patch('kuryr_kubernetes.controller.drivers.base.'
                 'ServiceSecurityGroupsDriver.get_instance')
     @mock.patch('kuryr_kubernetes.controller.drivers.base.'
@@ -464,10 +552,14 @@ class TestKuryrLoadBalancerHandler(test_base.TestCase):
                 '.PodProjectDriver.get_instance')
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
                 '.LBaaSDriver.get_instance')
+    @mock.patch('kuryr_kubernetes.controller.drivers.base'
+                '.NodesSubnetsDriver.get_instance', mock.Mock())
     def test_sync_lbaas_members_udp(self, m_get_drv_lbaas,
                                     m_get_drv_project, m_get_drv_subnets,
-                                    m_k8s, m_svc_project_drv, m_svc_sg_drv):
+                                    m_k8s, m_svc_project_drv, m_svc_sg_drv,
+                                    m_get_cidr):
         # REVISIT(ivc): test methods separately and verify ensure/release
+        m_get_cidr.return_value = '10.0.0.128/26'
         project_id = str(uuid.uuid4())
         subnet_id = str(uuid.uuid4())
         expected_ip = "1.2.3.4"
@@ -485,6 +577,7 @@ class TestKuryrLoadBalancerHandler(test_base.TestCase):
         self.assertEqual(sorted(expected_targets.items()), observed_targets)
         self.assertEqual(expected_ip, str(crd['status']['loadbalancer']['ip']))
 
+    @mock.patch('kuryr_kubernetes.utils.get_subnet_cidr')
     @mock.patch('kuryr_kubernetes.controller.drivers.base.'
                 'ServiceSecurityGroupsDriver.get_instance')
     @mock.patch('kuryr_kubernetes.controller.drivers.base.'
@@ -496,10 +589,13 @@ class TestKuryrLoadBalancerHandler(test_base.TestCase):
                 '.PodProjectDriver.get_instance')
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
                 '.LBaaSDriver.get_instance')
+    @mock.patch('kuryr_kubernetes.controller.drivers.base'
+                '.NodesSubnetsDriver.get_instance', mock.Mock())
     def test_sync_lbaas_members_svc_listener_port_edit(
             self, m_get_drv_lbaas, m_get_drv_project, m_get_drv_subnets,
-            m_k8s, m_svc_project_drv, m_svc_sg_drv):
+            m_k8s, m_svc_project_drv, m_svc_sg_drv, m_get_cidr):
         # REVISIT(ivc): test methods separately and verify ensure/release
+        m_get_cidr.return_value = '10.0.0.128/26'
         project_id = str(uuid.uuid4())
         subnet_id = str(uuid.uuid4())
         expected_ip = '1.2.3.4'
@@ -523,6 +619,7 @@ class TestKuryrLoadBalancerHandler(test_base.TestCase):
 
         self.assertEqual(expected_ip, str(crd['status']['loadbalancer']['ip']))
 
+    @mock.patch('kuryr_kubernetes.utils.get_subnet_cidr')
     @mock.patch('kuryr_kubernetes.controller.drivers.base.'
                 'ServiceSecurityGroupsDriver.get_instance')
     @mock.patch('kuryr_kubernetes.controller.drivers.base.'
@@ -534,10 +631,13 @@ class TestKuryrLoadBalancerHandler(test_base.TestCase):
                 '.PodProjectDriver.get_instance')
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
                 '.LBaaSDriver.get_instance')
+    @mock.patch('kuryr_kubernetes.controller.drivers.base'
+                '.NodesSubnetsDriver.get_instance', mock.Mock())
     def test_add_new_members_udp(self, m_get_drv_lbaas,
                                  m_get_drv_project, m_get_drv_subnets,
                                  m_k8s, m_svc_project_drv,
-                                 m_svc_sg_drv):
+                                 m_svc_sg_drv, m_get_cidr):
+        m_get_cidr.return_value = '10.0.0.128/26'
         project_id = str(uuid.uuid4())
         subnet_id = str(uuid.uuid4())
         crd = get_lb_crd()
@@ -557,3 +657,41 @@ class TestKuryrLoadBalancerHandler(test_base.TestCase):
 
         self.assertEqual(member_added, False)
         m_drv_lbaas.ensure_member.assert_not_called()
+
+    @mock.patch('kuryr_kubernetes.clients.get_kubernetes_client')
+    @mock.patch('kuryr_kubernetes.controller.drivers.base'
+                '.LBaaSDriver.get_instance')
+    def test_reconcile_loadbalancers(self, m_get_drv_lbaas, m_k8s):
+        loadbalancer_crds = get_lb_crds()
+        m_handler = mock.MagicMock(spec=h_lb.KuryrLoadBalancerHandler)
+        m_handler._drv_lbaas = m_get_drv_lbaas
+        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
+        lbaas_spec = {}
+        lbaas.load_balancers.return_value = []
+
+        selflink = ['/apis/openstack.org/v1/namespaces/default/'
+                    'kuryrloadbalancers/test',
+                    '/apis/openstack.org/v1/namespaces/default/'
+                    'kuryrloadbalancers/demo']
+        h_lb.KuryrLoadBalancerHandler._trigger_loadbalancer_reconciliation(
+            m_handler, loadbalancer_crds)
+        lbaas.load_balancers.assert_called_once_with(**lbaas_spec)
+        m_handler._reconcile_lbaas.assert_called_with(selflink)
+
+    @mock.patch('kuryr_kubernetes.clients.get_kubernetes_client')
+    @mock.patch('kuryr_kubernetes.controller.drivers.base'
+                '.LBaaSDriver.get_instance')
+    def test_reconcile_loadbalancers_in_sync(self, m_get_drv_lbaas, m_k8s):
+        loadbalancer_crds = get_lb_crds()
+
+        m_handler = mock.MagicMock(spec=h_lb.KuryrLoadBalancerHandler)
+        m_handler._drv_lbaas = m_get_drv_lbaas
+        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
+        loadbalancers = [{'id': '01234567890'}, {'id': '01234567891'}]
+        lbaas_spec = {}
+        lbaas.load_balancers.return_value = loadbalancers
+
+        h_lb.KuryrLoadBalancerHandler._trigger_loadbalancer_reconciliation(
+            m_handler, loadbalancer_crds)
+        lbaas.load_balancers.assert_called_once_with(**lbaas_spec)
+        m_handler._reconcile_lbaas.assert_not_called()
