@@ -16,7 +16,6 @@
 from unittest import mock
 
 from os_vif import objects as os_obj
-from oslo_serialization import jsonutils
 
 from kuryr_kubernetes import constants as k_const
 from kuryr_kubernetes.controller.drivers import base as drivers
@@ -24,6 +23,7 @@ from kuryr_kubernetes.controller.handlers import vif as h_vif
 from kuryr_kubernetes import exceptions as k_exc
 from kuryr_kubernetes.objects import vif
 from kuryr_kubernetes.tests import base as test_base
+from kuryr_kubernetes.tests import fake
 
 
 class TestVIFHandler(test_base.TestCase):
@@ -43,18 +43,12 @@ class TestVIFHandler(test_base.TestCase):
 
         self._pod_version = mock.sentinel.pod_version
         self._pod_link = mock.sentinel.pod_link
-        self._pod_namespace = mock.sentinel.namespace
+        self._pod_namespace = 'namespace1'
         self._pod_uid = mock.sentinel.pod_uid
         self._pod_name = 'pod1'
-        self._pod = {
-            'metadata': {'resourceVersion': self._pod_version,
-                         'selfLink': self._pod_link,
-                         'name': self._pod_name,
-                         'namespace': self._pod_namespace},
-            'status': {'phase': k_const.K8S_POD_STATUS_PENDING},
-            'spec': {'hostNetwork': False,
-                     'nodeName': 'hostname'}
-        }
+        self._pod = fake.get_k8s_pod()
+        self._pod['status'] = {'phase': k_const.K8S_POD_STATUS_PENDING}
+        self._pod['spec'] = {'hostNetwork': False, 'nodeName': 'hostname'}
 
         self._kp_version = mock.sentinel.kp_version
         self._kp_link = mock.sentinel.kp_link
@@ -84,7 +78,6 @@ class TestVIFHandler(test_base.TestCase):
         self._request_vif = self._handler._drv_vif_pool.request_vif
         self._release_vif = self._handler._drv_vif_pool.release_vif
         self._activate_vif = self._handler._drv_vif_pool.activate_vif
-        self._matc = self._handler._move_annotations_to_crd
         self._is_pod_scheduled = self._handler._is_pod_scheduled
         self._is_pod_completed = self._handler._is_pod_completed
         self._request_additional_vifs = \
@@ -100,48 +93,6 @@ class TestVIFHandler(test_base.TestCase):
         self._set_vifs_driver.return_value = mock.Mock(
             spec=drivers.PodVIFDriver)
 
-    @mock.patch.object(h_vif.VIFHandler, '_is_network_policy_enabled')
-    @mock.patch.object(drivers.MultiVIFDriver, 'get_enabled_drivers')
-    @mock.patch.object(drivers.VIFPoolDriver, 'set_vif_driver')
-    @mock.patch.object(drivers.VIFPoolDriver, 'get_instance')
-    @mock.patch.object(drivers.PodVIFDriver, 'get_instance')
-    @mock.patch.object(drivers.PodSecurityGroupsDriver, 'get_instance')
-    @mock.patch.object(drivers.PodSubnetsDriver, 'get_instance')
-    @mock.patch.object(drivers.PodProjectDriver, 'get_instance')
-    @mock.patch.object(drivers.LBaaSDriver, 'get_instance')
-    @mock.patch.object(drivers.ServiceSecurityGroupsDriver, 'get_instance')
-    def test_init(self, m_get_svc_sg_driver, m_get_lbaas_driver,
-                  m_get_project_driver, m_get_subnets_driver, m_get_sg_driver,
-                  m_get_vif_driver, m_get_vif_pool_driver, m_set_vifs_driver,
-                  m_get_multi_vif_drivers, m_is_network_policy_enabled):
-        project_driver = mock.sentinel.project_driver
-        subnets_driver = mock.sentinel.subnets_driver
-        sg_driver = mock.sentinel.sg_driver
-        vif_driver = mock.sentinel.vif_driver
-        vif_pool_driver = mock.Mock(spec=drivers.VIFPoolDriver)
-        multi_vif_drivers = [mock.MagicMock(spec=drivers.MultiVIFDriver)]
-        lbaas_driver = mock.sentinel.lbaas_driver
-        svc_sg_driver = mock.Mock(spec=drivers.ServiceSecurityGroupsDriver)
-        m_get_project_driver.return_value = project_driver
-        m_get_subnets_driver.return_value = subnets_driver
-        m_get_sg_driver.return_value = sg_driver
-        m_get_vif_driver.return_value = vif_driver
-        m_get_vif_pool_driver.return_value = vif_pool_driver
-        m_get_multi_vif_drivers.return_value = multi_vif_drivers
-        m_get_lbaas_driver.return_value = lbaas_driver
-        m_get_svc_sg_driver.return_value = svc_sg_driver
-        m_is_network_policy_enabled.return_value = True
-
-        handler = h_vif.VIFHandler()
-
-        self.assertEqual(project_driver, handler._drv_project)
-        self.assertEqual(subnets_driver, handler._drv_subnets)
-        self.assertEqual(sg_driver, handler._drv_sg)
-        self.assertEqual(vif_pool_driver, handler._drv_vif_pool)
-        self.assertEqual(multi_vif_drivers, handler._drv_multi_vif)
-        self.assertEqual(lbaas_driver, handler._drv_lbaas)
-        self.assertEqual(svc_sg_driver, handler._drv_svc_sg)
-
     def test_is_pod_scheduled(self):
         self.assertTrue(h_vif.VIFHandler._is_pod_scheduled(self._pod))
 
@@ -175,14 +126,12 @@ class TestVIFHandler(test_base.TestCase):
                                      m_get_k8s_client):
         m_get_kuryrport.return_value = self._kp
         m_host_network.return_value = True
-        self._matc.return_value = False
         k8s = mock.MagicMock()
         m_get_k8s_client.return_value = k8s
 
         h_vif.VIFHandler.on_present(self._handler, self._pod)
 
         k8s.add_finalizer.assert_not_called()
-        self._matc.assert_not_called()
         m_get_kuryrport.assert_not_called()
         self._request_vif.assert_not_called()
         self._request_additional_vifs.assert_not_called()
@@ -196,46 +145,23 @@ class TestVIFHandler(test_base.TestCase):
         m_get_kuryrport.return_value = self._kp
         m_host_network.return_value = False
         self._is_pod_scheduled.return_value = False
-        self._matc.return_value = False
         k8s = mock.MagicMock()
         m_get_k8s_client.return_value = k8s
 
         h_vif.VIFHandler.on_present(self._handler, self._pod)
 
         k8s.add_finalizer.assert_not_called()
-        self._matc.assert_not_called()
         m_get_kuryrport.assert_not_called()
         self._request_vif.assert_not_called()
         self._request_additional_vifs.assert_not_called()
         self._activate_vif.assert_not_called()
 
-    @mock.patch('kuryr_kubernetes.clients.get_kubernetes_client')
-    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_kuryrport')
-    def test_on_present_on_completed_with_annotation(self, m_get_kuryrport,
-                                                     m_get_k8s_client):
-        self._is_pod_completed.return_value = True
-        m_get_kuryrport.return_value = self._kp
-        self._matc.return_value = False
-        k8s = mock.MagicMock()
-        m_get_k8s_client.return_value = k8s
-
-        h_vif.VIFHandler.on_present(self._handler, self._pod)
-
-        k8s.add_finalizer.assert_called_once_with(self._pod,
-                                                  k_const.POD_FINALIZER)
-        self._matc.assert_called_once_with(self._pod)
-        self._handler.on_finalize.assert_called_once_with(self._pod)
-        self._request_vif.assert_not_called()
-        self._request_additional_vifs.assert_not_called()
-        self._activate_vif.assert_not_called()
-
     @mock.patch('kuryr_kubernetes.clients.get_kubernetes_client')
     @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_kuryrport')
     def test_on_present_on_completed_without_annotation(self, m_get_kuryrport,
                                                         m_get_k8s_client):
         self._is_pod_completed.return_value = True
         m_get_kuryrport.return_value = None
-        self._matc.return_value = False
         k8s = mock.MagicMock()
         m_get_k8s_client.return_value = k8s
 
@@ -243,7 +169,6 @@ class TestVIFHandler(test_base.TestCase):
 
         k8s.add_finalizer.assert_called_once_with(self._pod,
                                                   k_const.POD_FINALIZER)
-        self._matc.assert_called_once_with(self._pod)
         self._handler.on_finalize.assert_not_called()
         self._request_vif.assert_not_called()
         self._request_additional_vifs.assert_not_called()
@@ -256,7 +181,6 @@ class TestVIFHandler(test_base.TestCase):
                                m_get_k8s_client):
         m_get_kuryrport.return_value = None
         m_host_network.return_value = False
-        self._matc.return_value = False
         k8s = mock.MagicMock()
         m_get_k8s_client.return_value = k8s
 
@@ -265,7 +189,6 @@ class TestVIFHandler(test_base.TestCase):
         k8s.add_finalizer.assert_called_once_with(self._pod,
                                                   k_const.POD_FINALIZER)
         m_get_kuryrport.assert_called_once_with(self._pod)
-        self._matc.assert_called_once_with(self._pod)
         self._handler._add_kuryrport_crd.assert_called_once_with(self._pod)
 
     @mock.patch('kuryr_kubernetes.clients.get_kubernetes_client')
@@ -275,7 +198,6 @@ class TestVIFHandler(test_base.TestCase):
                                m_get_k8s_client):
         m_get_kuryrport.return_value = self._kp
         m_host_network.return_value = False
-        self._matc.return_value = False
         k8s = mock.MagicMock()
         m_get_k8s_client.return_value = k8s
 
@@ -283,7 +205,6 @@ class TestVIFHandler(test_base.TestCase):
 
         k8s.add_finalizer.assert_called_once_with(self._pod,
                                                   k_const.POD_FINALIZER)
-        self._matc.assert_called_once_with(self._pod)
         m_get_kuryrport.assert_called_once_with(self._pod)
         self._handler._add_kuryrport_crd.assert_not_called()
 
@@ -294,7 +215,6 @@ class TestVIFHandler(test_base.TestCase):
                                 m_get_k8s_client):
         m_get_kuryrport.return_value = self._kp
         m_host_network.return_value = False
-        self._matc.return_value = True
         k8s = mock.MagicMock()
         m_get_k8s_client.return_value = k8s
 
@@ -302,8 +222,7 @@ class TestVIFHandler(test_base.TestCase):
 
         k8s.add_finalizer.assert_called_once_with(self._pod,
                                                   k_const.POD_FINALIZER)
-        self._matc.assert_called_once_with(self._pod)
-        m_get_kuryrport.assert_not_called()
+        m_get_kuryrport.assert_called()
         self._request_vif.assert_not_called()
         self._request_additional_vifs.assert_not_called()
         self._activate_vif.assert_not_called()
@@ -313,7 +232,6 @@ class TestVIFHandler(test_base.TestCase):
     def test_on_present_pod_finalizer_exception(self, m_host_network,
                                                 m_get_k8s_client):
         m_host_network.return_value = False
-        self._matc.return_value = True
         k8s = mock.MagicMock()
         k8s.add_finalizer.side_effect = k_exc.K8sClientException
         m_get_k8s_client.return_value = k8s
@@ -373,29 +291,3 @@ class TestVIFHandler(test_base.TestCase):
                 crd=self._pod["metadata"]["name"]))
         (k8s.remove_finalizer
          .assert_called_once_with(self._pod, k_const.POD_FINALIZER))
-
-    def test_move_annotations_to_crd_no_annotations(self):
-        res = h_vif.VIFHandler._move_annotations_to_crd(self._handler,
-                                                        self._pod)
-        self.assertFalse(res)
-
-    @mock.patch('kuryr_kubernetes.clients.get_kubernetes_client')
-    def test_move_annotations_to_crd_with_annotations(self, m_get_k8s_client):
-        vifobj = os_obj.vif.VIFOpenVSwitch()
-        state = vif.PodState(default_vif=vifobj)
-        annotation = jsonutils.dumps(state.obj_to_primitive())
-        self._pod['metadata']['annotations'] = {
-            k_const.K8S_ANNOTATION_VIF: annotation}
-        vifs = {'eth0': {'default': True, 'vif': vifobj.obj_to_primitive()}}
-        k8s = mock.MagicMock()
-        m_get_k8s_client.return_value = k8s
-
-        res = h_vif.VIFHandler._move_annotations_to_crd(self._handler,
-                                                        self._pod)
-        self.assertTrue(res)
-        self._handler._add_kuryrport_crd.assert_called_once_with(self._pod,
-                                                                 vifs)
-
-        m_get_k8s_client.assert_called_once()
-        k8s.remove_annotations.assert_called_once_with(
-            self._pod['metadata']['selfLink'], k_const.K8S_ANNOTATION_VIF)
